// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.7 $
#include <hk_class.h>
#include <termios.h>
#include <sys/stat.h>

hk_string hk_class::p_begintag_begin="<";
hk_string hk_class::p_begintag_end=">";
hk_string hk_class::p_endtag_begin="</";
hk_string hk_class::p_endtag_end=">";
hk_string hk_class::p_begintag="";
hk_string hk_class::p_endtag="";
hk_string hk_class::p_defaulttimeformat="h:m:s";
hk_string hk_class::p_defaultdateformat="D.M.Y";
hk_string hk_class::p_defaultdatetimeformat="D.M.Y h:m:s";


messagewindowtype* hk_class::p_warning=NULL;
yesno_dialogtype* hk_class::p_yesno=NULL;
translatefunctiontype* hk_class::p_translate=NULL;

bool 	hk_class::p_showpedantic=true;
bool 	hk_class::p_generaldebug=false;
bool 	hk_class::p_runtime_only=false;

stringvalue_dialogtype* hk_class::p_stringvalue=NULL;

hk_string line="========================================\n";

hk_class::hk_class()
{
#ifdef HK_DEBUG
 	p_debug=false;
#endif
}

hk_class::~hk_class()
{
#ifdef HK_DEBUG
hkdebug("hk_class::destructor");
#endif
}

#ifdef HK_DEBUG
void hk_class::hkclassname(const hk_string& n)
{
#ifdef HK_DEBUG
hkdebug("hk_class::classname");
#endif

p_classname=n;
}

hk_string hk_class::hkclassname(void)
{
 return p_classname;

}
void hk_class::wanna_debug(bool d)
{
	p_debug=d;
}

void hk_class::hkdebug(const hk_string& d)
{
	if (p_debug||p_generaldebug)
	{
    		cout <<"HKDebug: ";
    		if (p_classname.size()>0) cout<<p_classname;
    		else cout<<"Unknown classname";
    		cout<<" "<<d<<endl;
    	}

}

void hk_class::hkdebug(const hk_string& d, const hk_string& e)
{
	if (p_debug||p_generaldebug)
	{
    		cout <<"HKDebug: ";
    		if (p_classname.size()>0) cout<<p_classname;
    		else cout<<"Unknown classname";
    		cout<<" "<<d<<e<<endl;
    	}

}
void hk_class::hkdebug(const hk_string& d,int i)
{
	if (p_debug||p_generaldebug)
	{
    		cout <<"HKDebug: ";
    		if (p_classname.size()>0) cout<<p_classname;
    		else cout<<"Unknown classname";
    		cout<<" "<<d<<i<<endl;
    	}

}
void hk_class::hkdebug(const hk_string& d,double i)
{
	if (p_debug||p_generaldebug)
	{
    		cout <<"HKDebug: ";
    		if (p_classname.size()>0) cout<<p_classname;
    		else cout<<"Unknown classname";
    		cout<<" "<<d<<i<<endl;
    	}

}

#endif


void hk_class::show_warningmessage(const hk_string& m)
{
    if (p_warning!=NULL)p_warning(m);
    else cerr <<m<<endl;
}

void hk_class::set_warningmessage(messagewindowtype* m)
{
    hk_class::p_warning=m;
}

void hk_class::set_translatefunction(translatefunctiontype* t)
{
   hk_class::p_translate=t;
}

bool hk_class::show_yesnodialog(const hk_string& m, bool default_value)
{
    if (!p_showpedantic)return default_value;
    if (p_yesno!=NULL) return p_yesno(m,default_value);
    cerr <<"\n"<<line<<hk_translate("Question:")<<endl<<line;
    cerr <<m<<endl;
    cerr <<hk_translate("n=No            all other keys = yes")<<endl<<line;

    hk_string i;
    cin >>i;
    if (i=="n"||i=="N") return false;
    else return true;


}


void hk_class::set_yesnodialog(yesno_dialogtype* d)
{

  p_yesno=d;
}

void hk_class::set_showpedantic(bool s)
{
 p_showpedantic=s;
}

#ifdef HK_DEBUG
void hk_class::set_generaldebug(bool d)
{

   p_generaldebug=d;
}
#endif


hk_string hk_class::hk_translate(const hk_string& t)
{
 if (p_translate==NULL) return t;
 else return p_translate(t);

}


hk_string  hk_class::show_stringvaluedialog(const hk_string& t)
{

 if (p_stringvalue!=NULL) return p_stringvalue(t);

 cerr <<endl<< t<<endl;
 hk_string i;
 cin >>i;
 return i;
}

void hk_class::set_stringvaluedialog(stringvalue_dialogtype* t)
{

  p_stringvalue=t;
}
void  hk_class::savedata(ofstream& s)
{
}

void  hk_class::loaddata(const hk_string& definition)
{

}

bool hk_class::get_tagvalue(const hk_string &where,const hk_string &tag, hk_string &value, int position=1)
{
  unsigned long beginposition=0;
  unsigned long endposition=0;
  int counter=0;
  set_tag(tag);
  while(counter<position)
  {
    beginposition=where.find(p_begintag,beginposition);
    if (beginposition>where.size())return false;
    beginposition+=p_begintag.size();
    counter++;
  }
  bool res=true;
  endposition=where.find(p_endtag,beginposition);
  if (endposition>where.size())res=false;
  value.assign(where,beginposition,endposition-beginposition);
  value=replace_all("!#!",value,"<");

return res;

}

bool hk_class::get_tagvalue(const hk_string &where, const hk_string &tag, unsigned long &value, int position=1)
{
 hk_string v;
 bool r=get_tagvalue(where,tag,v,position);
 if (r) value=atoi(v.c_str());
 return r;
}

bool hk_class::get_tagvalue(const hk_string &where, const hk_string &tag, long &value, int position=1)
{
 hk_string v;
 bool r=get_tagvalue(where,tag,v,position);
 if (r) value=atoi(v.c_str());
 return r;
}

bool hk_class::get_tagvalue(const hk_string &where, const hk_string &tag, bool &value, int position=1)
{
 hk_string v;
 bool r=get_tagvalue(where,tag,v,position);
 if (r) value=(v=="YES");

 return r;

}

void hk_class::set_tagvalue(ofstream& stream,const hk_string& tag,const hk_string& value)
{
#ifdef HK_DEBUG
//  cout <<"hk_class::set_tagvalue "<<tag<<endl;
//  p_generaldebug=true;
#endif

  set_tag(tag);
  hk_string v=replace_all("<",value,"!#!");
  stream << p_begintag<<v<<p_endtag<<endl;
}

void hk_class::set_tagvalue(ofstream& stream,const hk_string& tag,unsigned long value)
{
  set_tag(tag);
  stream << p_begintag<<value<<p_endtag<<endl;
}

void hk_class::set_tagvalue(ofstream& stream,const hk_string& tag,long value)
{
  set_tag(tag);
  stream << p_begintag<<value<<p_endtag<<endl;
}

void hk_class::set_tagvalue(ofstream& stream,const hk_string& tag,bool value)
{
  set_tag(tag);
  stream << p_begintag<<(value ?"YES":"NO")<<p_endtag<<endl;
}

void hk_class::set_tag(const hk_string& tag)
{
   p_begintag=p_begintag_begin+tag+p_begintag_end;
   p_endtag=p_endtag_begin+tag+p_endtag_end;
}

void hk_class::start_mastertag(ofstream& stream,const hk_string& tag)
{
 if (tag.size()>0) stream <<p_begintag_begin<<tag<<p_begintag_end<<endl;
}

void hk_class::end_mastertag(ofstream& stream,const hk_string & tag)
{
 if (tag.size()>0) stream <<p_endtag_begin<<tag<<p_endtag_end<<endl<<endl;

}

hk_string hk_class::defaultdateformat(void)
{
   return   p_defaultdateformat;
}

hk_string hk_class::defaulttimeformat(void)
{
   return p_defaulttimeformat;
}

hk_string hk_class::defaultdatetimeformat(void)
{
  return p_defaultdatetimeformat;
}

void hk_class::set_defaultdatetimeformat(const hk_string& tformat,const hk_string& dformat,const hk_string&dtformat)
{
   p_defaulttimeformat=tformat;
   p_defaultdateformat=dformat;
   p_defaultdatetimeformat=dtformat;

}



void hk_class::save_preferences(void)
{
  hk_string p_classespath=getenv("HOME");
  p_classespath +="/.hk_classes";
  mkdir (p_classespath.c_str(),S_IRUSR|S_IWUSR|S_IXUSR);
  hk_string p_filename =p_classespath+"/preferences";
  ofstream* s= new ofstream(p_filename.c_str());
  if (s==NULL)return;
  start_mastertag(*s,"HK_REGIONAL");
  set_tagvalue(*s,"DEFAULTTIMEFORMAT",p_defaulttimeformat);
  set_tagvalue(*s,"DEFAULTDATETIMEFORMAT",p_defaultdatetimeformat);
  set_tagvalue(*s,"DEFAULTDATEFORMAT",p_defaultdateformat);
  end_mastertag(*s,"HK_REGIONAL");
  s->close();


}

bool hk_class::runtime_only(void)
{
   return p_runtime_only;
}

