// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.16 $
#include <hk_column.h>
#include <hk_datasource.h>
#include "hk_dsdatavisible.h"
#include <stdio.h>
#include <hk_datetime.h>



hk_column::hk_column(hk_datasource* ds, const hk_string& lTRUE,const hk_string& lFALSE)
{
#ifdef HK_DEBUG
   hkdebug("hk_column::hk_column");
#endif
	p_datasource=ds;
	p_size=2;
	p_fieldtype=integerfield;
        p_primary_index=false;
        p_notnull=false;
        set_has_not_changed();
        p_readonly=false;
        p_sql_column_delimiter="\"";
        p_blob.length=0;
        p_blob.data=NULL;
        p_driver_specific_data=NULL;
        p_original_new_data=NULL;
        p_driver_specific_data_size=0;
        p_original_new_data_size=0;
	set_boolvalues(lTRUE,lFALSE);
        p_dateformat=p_defaultdateformat;
        p_timeformat=p_defaulttimeformat;
        p_datetimeformat=p_defaultdatetimeformat;
        p_driverspecific_dateformat="Y-M-D";
        p_driverspecific_timeformat="h:m:s";
        p_driverspecific_datetimeformat="Y-M-D h:m:s";
	p_internal=NULL;

}


hk_column::~hk_column()
{
#ifdef HK_DEBUG
hkdebug("hk_column::~hk_column");
#endif
  	p_datasource->column_remove(this);
if (p_driver_specific_data!=NULL) delete[] p_driver_specific_data ;
p_driver_specific_data=NULL;
if (p_original_new_data!=NULL) delete[] p_original_new_data;
p_original_new_data=NULL;
}


void hk_column::set_name(const hk_string& n)
{
#ifdef HK_DEBUG
hkdebug("hk_column::set_name");
#endif
    	p_fieldname=replace_all(" ",n,"_");
}



hk_string hk_column::name(void)
{
	return p_fieldname;
}



bool hk_column::is_readonly(void)
{
  if (p_datasource->is_readonly()) return true;
  if (p_fieldtype==auto_incfield) return true;
  return p_readonly;
}



void hk_column::set_readonly(bool r)
{
#ifdef HK_DEBUG
hkdebug("hk_column::set_readonly");
#endif
	if ((p_datasource->mode()==hk_datasource::mode_disabled)||
	    (p_datasource->mode()==hk_datasource::mode_createtable)||
	    (p_datasource->mode()==hk_datasource::mode_altertable)
	   )
 	p_readonly=r;
}




void	hk_column::set_fieldtype(enum_fieldtype f)
{
#ifdef HK_DEBUG
hkdebug("hk_column::set_fieldtype");
#endif
	if ((p_datasource->mode()==hk_datasource::mode_disabled)||
	    (p_datasource->mode()==hk_datasource::mode_createtable)||
	    (p_datasource->mode()==hk_datasource::mode_altertable)
	   )
   	 p_fieldtype=f;
}



hk_column::enum_fieldtype	hk_column::fieldtype( void)
{
    	return p_fieldtype;
}



bool hk_column::is_primary(void)
{
 return p_primary_index;

}



bool hk_column::set_primary(bool i)
{
#ifdef HK_DEBUG
hkdebug("hk_column::set_primary");
#endif

	if ((p_datasource->mode()==hk_datasource::mode_disabled)||
	    (p_datasource->mode()==hk_datasource::mode_createtable)||
	    (p_datasource->mode()==hk_datasource::mode_altertable)
	   )
	 {
	   	p_primary_index=i;
  		return true;
  	 }
  	else


  	return false;
}



bool hk_column::is_notnull(void)
{
 return p_notnull;

}



bool hk_column::set_notnull(bool i)
{
#ifdef HK_DEBUG
hkdebug("hk_column::set_notnull");
#endif
	if ((p_datasource->mode()==hk_datasource::mode_disabled)||
	    (p_datasource->mode()==hk_datasource::mode_createtable)||
	    (p_datasource->mode()==hk_datasource::mode_altertable)
	   )
	 {
	   	p_notnull=i;
  		return true;
  	 }
  	else return false;
}



long hk_column::size(void)
{

 	return p_size;

}



void hk_column::set_size(long s)
{
#ifdef HK_DEBUG
hkdebug("hk_column::set_size");
#endif
	if ((p_datasource->mode()==hk_datasource::mode_disabled)||
	    (p_datasource->mode()==hk_datasource::mode_createtable)||
	    (p_datasource->mode()==hk_datasource::mode_altertable)
	   )
  	p_size=s;
}




hk_string hk_column::asstring(void)
{
   if(p_datasource->is_enabled()&&(p_datasource->max_rows()>0))
   {
     return asstring_at(p_datasource->row_position());
   }
  	else return "";
}





hk_string hk_column::changed_data_asstring(void)
{
if (p_has_changed) return p_new_data_asstring;
else return "";
}



hk_string hk_column::fieldtype_name()
{
	                switch(p_fieldtype)
	                {
	                	case floatingfield :      return 	hk_translate("Float");
	                	case smallfloatingfield : return 	hk_translate("Small Float");
	                	case integerfield  :	     return  	hk_translate("Integer");
	                	case smallintegerfield  :    return  	hk_translate("Small Integer");
	                	case textfield     :	     return 	hk_translate("Text");
	                	case datetimefield :      return  	hk_translate("Datetime");
	                	case timefield     :      return  	hk_translate("Time");
	                	case datefield     :      return   	hk_translate("Date");
	                	case boolfield     :      return   	hk_translate("Bool");
	                	case binaryfield     :      return   	hk_translate("Binary");
	                	case memofield     :      return   	hk_translate("Memo");
	                	case auto_incfield :      return    	hk_translate("Auto Increment");
	                	default 	      :      return  	hk_translate("Unknown Fieldtype");

	                }

}



bool hk_column::has_changed(void)
{
 return p_has_changed;
}



void hk_column::set_asstring(const hk_string& s,bool registerchange)
{

#ifdef HK_DEBUG
 hkdebug("hk_column::set_asstring(hk_string s)");
#endif
if (is_readonly())return;

hk_string buf=s;
if (p_fieldtype==datefield) buf=transfer_date(buf.c_str(),p_dateformat.c_str(),p_driverspecific_dateformat.c_str());
if (p_fieldtype==timefield) buf=transfer_time(buf.c_str(),p_timeformat.c_str(),p_driverspecific_timeformat.c_str());
if (p_fieldtype==datetimefield) buf=transfer_datetime(buf.c_str(),p_datetimeformat.c_str(),p_driverspecific_datetimeformat.c_str());
if (p_driver_specific_data!=NULL)
	{
		delete[] p_driver_specific_data;
		p_driver_specific_data=NULL;
	}
 if (p_datasource->is_enabled())
 {
   if(driver_specific_asstring(buf.c_str()))
   	{
		if (registerchange) set_has_changed();
		p_new_data_asstring=buf.c_str();

	#ifdef HK_DEBUG
 	 hkdebug("column: data marked as changed");
	#endif

   	}
   else
   {
	#ifdef HK_DEBUG
 	 hkdebug("column: data NOT marked as changed");
	#endif
   }


	#ifdef HK_DEBUG
 	 hkdebug("datasource enabled => data update");
 	 if (p_has_changed)hkdebug("p_datachanged = true"); else hkdebug("p_datachanged = false");
	 hkdebug(s);
	#endif

 }
 else
 {
#ifdef HK_DEBUG
 hkdebug("datasource not enabled => no data update");
#endif
 }

}



hk_string hk_column::sql_column_delimiter(void)
{
  return p_sql_column_delimiter;
}



hk_string hk_column::asstring_at(unsigned long a)
{
	if (p_fieldtype==binaryfield)return hk_translate("Binary");
	if (p_datasource->is_enabled()&&(p_datasource->max_rows()>0)&&(a<p_datasource->max_rows()))
	{
	if (p_fieldtype==datefield) return transfer_date(driver_specific_asstring_at(a),p_driverspecific_dateformat,p_dateformat);
	if (p_fieldtype==timefield) return transfer_time(driver_specific_asstring_at(a),p_driverspecific_timeformat.c_str(),p_timeformat.c_str());
	if (p_fieldtype==datetimefield) return transfer_datetime(driver_specific_asstring_at(a),p_driverspecific_datetimeformat.c_str(),p_datetimeformat.c_str());
	  return driver_specific_asstring_at(a);
	}
	else return "";
}



const char* hk_column::transformed_asstring(void)
{
 	return driver_specific_transformed_asstring_at(p_datasource->row_position());
}

const char* hk_column::transformed_asstring_at(unsigned long position)
{
 	return driver_specific_transformed_asstring_at(position);
}



const struct_raw_data* hk_column::asblob(void)
{
 	return driver_specific_asblob();
}



void hk_column::set_asblob(const struct_raw_data* rd)
{
#ifdef HK_DEBUG
hkdebug("hk_column::set_asblob");
#endif
if (is_readonly())return;

  if (rd==NULL)return;
   if (p_original_new_data!=NULL)
  	{
 	  delete[] p_original_new_data;
 	  p_original_new_data=NULL;
 	}
  p_original_new_data=new char[rd->length];
  p_original_new_data_size=rd->length;
  memcpy(p_original_new_data,rd->data,rd->length);
  driver_specific_asblob(p_original_new_data);
  set_has_changed();


}



const struct_raw_data* hk_column::asblob_at(unsigned long a)
{
 	return driver_specific_asblob_at(a);
}



const struct_raw_data* hk_column::transformed_changed_data(void)
{
  p_transformed_changed_data_struct.length=p_driver_specific_data_size;
  p_transformed_changed_data_struct.data=p_driver_specific_data;
  return &p_transformed_changed_data_struct;
}



const struct_raw_data* hk_column::changed_data(void)
{
  p_changed_data_struct.length=p_original_new_data_size;
  p_changed_data_struct.data=p_original_new_data;
  return &p_changed_data_struct;

}



unsigned int hk_column::fieldnumber(void)
{
   return p_fieldnr;
}


void hk_column::reset_changed_data(void)
{
#ifdef HK_DEBUG
hkdebug("hk_column::reset_changed_data");
#endif
 set_has_not_changed();
 p_original_new_data_size=0;
 if (p_original_new_data!=NULL)delete[] p_original_new_data;
 p_original_new_data=NULL;
 p_driver_specific_data_size=0;
 if (p_driver_specific_data!=NULL)delete[] p_driver_specific_data;
 p_driver_specific_data=NULL;

}


void hk_column::set_asbool(bool b)
{
#ifdef HK_DEBUG
hkdebug("hk_column::set_asbool");
#endif
if (is_readonly())return;

 driver_specific_asbool(b);
 set_has_changed();

}


bool hk_column::asbool(void)
{
 return driver_specific_asbool();
}


bool hk_column::asbool_at(unsigned long a)
{
   return driver_specific_asbool_at(a);
}


bool hk_column::changed_data_asbool(void)
{
 if (p_new_data_asstring==p_true) return true;
 	else return false;


}


void hk_column::set_boolvalues(const hk_string& TRUE,const hk_string& FALSE)
{
	if (TRUE==FALSE)return;
	 p_true=TRUE;
	 p_false=FALSE;
}

hk_string hk_column::bool_truevalue(void)
{
    return p_true;
}

hk_string hk_column::bool_falsevalue(void)
{
    return p_false;
}


unsigned int hk_column::find(unsigned int from,unsigned int to,const hk_string& searchtext,bool wholephrase,bool casesensitive,bool backwards)
{
#ifdef HK_DEBUG
hkdebug("hk_column::find(unsigned int from,unsigned int to,conr hk_string& searchtext");
#endif
 unsigned int p_from, p_to, p_buffer;
 if (p_datasource->max_rows()==0)return 1;
 if (from <p_datasource->max_rows())p_from=from; else p_from=0;
 if (to <p_datasource->max_rows()) p_to=to;else p_to=p_datasource->max_rows()-1;
 if (p_from>p_to)
   {
      p_buffer=p_from;
      p_from=p_to;
      p_to=p_buffer;
   }
 if (backwards)
    {
      p_buffer=p_from;
      p_from=p_to;
      p_to=p_buffer;
    }
 for (unsigned int k=p_from;(backwards?((k>=p_to)&&(k<=p_from)):k<=p_to);(backwards?k--:k++))
 {
    if (is_findstring(k,searchtext,wholephrase,casesensitive))return k;
 }
 return p_datasource->max_rows()+1;
}



bool hk_column::is_findstring(unsigned int pos,const hk_string& searchtext,bool wholephrase,bool casesensitive)
{
#ifdef HK_DEBUG
hkdebug("hk_column::is_findstring");
#endif
 hk_string search=searchtext;
 hk_string where=asstring_at(pos);

  if (!casesensitive)
  {
	for (unsigned int tt=0;tt<where.size();tt++)
 	{
 	 where[tt]=toupper(where[tt]);
 	}
	for (unsigned int tt=0;tt<search.size();tt++)
 	{
 	 search[tt]=toupper(search[tt]);
 	}
  }
  if (wholephrase)
        {
	  if (where==search)return true;
	}
  else
        {
	  int a=where.find(search);
	  if (a>-1) return true;
	}

  return false;

}

unsigned int hk_column::find(const hk_string& searchtext,bool wholephrase,bool casesensitive,bool backwards)
{
#ifdef HK_DEBUG
hkdebug("hk_column::find(const hk_string& searchtext");
#endif
  return find(0,p_datasource->max_rows()-1,searchtext,wholephrase,casesensitive,backwards);
}



bool hk_column::load_from_file(const hk_string& filename)
{
#ifdef HK_DEBUG
hkdebug("hk_column::load_from_file");
#endif
   long size;
   FILE *in;
   in=fopen(filename.c_str(),"r");
   if (in==NULL)return false;
   fseek(in,0,SEEK_END);
   size=ftell(in);
   fseek(in,0,SEEK_SET);
   if (p_original_new_data!=NULL)
  	{
 	  delete[] p_original_new_data;
 	  p_original_new_data=NULL;
 	}
  p_original_new_data=new char[size];
  p_original_new_data_size=size;
  fread(p_original_new_data,size,1,in);
  driver_specific_asblob(p_original_new_data);
  set_has_changed();

return true;
}




bool hk_column::save_to_file(const hk_string& filename)
{
#ifdef HK_DEBUG
hkdebug("hk_column::save_to_file");
#endif

   FILE* out=fopen(filename.c_str(),"w");
   const struct_raw_data* res=asblob();
   if (res==NULL)return false;

  int r=fwrite(res->data,res->length,1,out);
   fclose(out);
   if (r<(signed)res->length)return false;
return true;
}


bool	hk_column::driver_specific_asstring(const hk_string& d)
{
   return true;
}


hk_string hk_column::driver_specific_asstring_at(unsigned long a)
{
   return "";
}


const struct_raw_data* hk_column::driver_specific_asblob_at(unsigned long a)
{
return NULL;
}


bool	    hk_column::driver_specific_asblob(const char* b)
{
return false;
}


void 	    hk_column::driver_specific_asbool(bool b)
{
}


bool hk_column::driver_specific_asbool_at(unsigned long a)
{
return false;
}

void hk_column::datavisible_add(hk_dsdatavisible* v)
{
#ifdef HK_DEBUG
hkdebug("hk_column::datavisible_add");
#endif
   p_datavisibles.insert(p_datavisibles.end(),v);
}


void hk_column::datavisible_remove(hk_dsdatavisible* v)
{
#ifdef HK_DEBUG
hkdebug("hk_column::datavisible_remove");
#endif
   p_datavisibles.remove(v);
}

void hk_column::data_has_changed(void)
{
#ifdef HK_DEBUG
hkdebug("hk_column::data_has_changed");
#endif
  list<hk_dsdatavisible*>::iterator it=p_datavisibles.begin();
  while (it!=p_datavisibles.end())
  {
   (*it)->columndata_has_changed();
   it++;
  }
}

void hk_column::set_has_changed()
{
#ifdef HK_DEBUG
hkdebug("hk_column::set_has_changed");
#endif
   p_has_changed=true;
   p_datasource->set_has_changed();
   data_has_changed();
}


void hk_column::set_has_not_changed()
{
#ifdef HK_DEBUG
hkdebug("hk_column::set_has_not_changed");
#endif
 p_has_changed=false;
}


