// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.8 $
#include <hk_connection.h>
#include <hk_database.h>
#include <time.h>
#include <termios.h>
#include <sys/stat.h>
#include <iostream.h>
#include "hk_drivermanager.h"
#include "hk_actionquery.h"

password_dialogtype* hk_connection::p_passworddialog=NULL;

hk_connection::hk_connection(hk_drivermanager* c)
{
	set_tcp_port(3306);
	p_connected=false;
 	p_connection_successfully_tried=false;
	p_database=NULL;
	p_debug_counter=0;
	p_host="localhost";
	p_user="root";
        srand(time(NULL));
	p_databaselist=new vector<hk_string>;
      p_classespath=getenv("HOME");
      p_classespath +="/.hk_classes";
      p_databasepath=p_classespath+"/";p_databasepath+=p_host;
      p_drivermanager=c;

}


hk_connection::~hk_connection()
{
#ifdef HK_DEBUG
hkdebug("hk_connection:destructor");
#endif
       if (p_database!=NULL)
         {
	   delete p_database;
	 }
        disconnect();
	if (p_drivermanager!=NULL) p_drivermanager->connection_remove(this);
        delete p_databaselist;
}

void hk_connection::set_password(const hk_string& p)
{
	p_password=p;
}


hk_string hk_connection::password(void)
{
 return p_password.c_str();
}


 void hk_connection::set_user(const hk_string& u)
{
	p_user=u;
}
hk_string hk_connection::user(void)
{
	return  p_user.c_str();;
}


void hk_connection::set_host(const hk_string& h)
{
	p_host=h;
    if (server_needs() &&  needs_host)
    {
      p_databasepath=p_classespath+"/";p_databasepath+=p_host;
      mkdir (p_databasepath.c_str(),S_IRUSR|S_IWUSR|S_IXUSR);
    }

}


hk_string hk_connection::host(void)
{
	return  p_host;
}

void hk_connection::set_tcp_port(unsigned int t)
{
	p_tcp_port=t;
;
}

unsigned int hk_connection::tcp_port(void)
{

 	return p_tcp_port;
}

bool  hk_connection::connect()
{
#ifdef HK_DEBUG
hkdebug("hk_connection::conncect");
#endif
     if  (p_connected)return true;
    if (p_connection_successfully_tried)
    {
 	if (driver_specific_connect())
 	{
 	  if (p_database!=NULL) p_database->select_db();
		return true;
 	}
 	else
 	{
 	show_warningmessage(hk_translate("Unable to connect to the SQL-Server!"));
 	return false;
 	}
     }
     else
     {
       bool cancel=false;
       while ((!is_connected())&&(!cancel))
       {
         if (!cancel) driver_specific_connect();
         if (!is_connected())cancel=show_passworddialog();
       }

     }
 	if (is_connected())
 	{
 	  if (p_database!=NULL) p_database->select_db();
		return true;
 	}
 	else
 	{
 	show_warningmessage(hk_translate("Unable to connect to the SQL-Server!"));
 	return false;
 	}

}
void hk_connection::set_passworddialog(password_dialogtype* d)
{
 p_passworddialog=d;
}
bool hk_connection::show_passworddialog(void)
{
#ifdef HK_DEBUG
hkdebug("hk_connection::show_passworddialog");
#endif
      struct_connectionfields fields;
      if (p_passworddialog!=NULL)
       {
         fields.host=host();
         fields.user=user();
	 fields.save=false;
         char* m= new char[50];
    	 sprintf(m,"%d",p_tcp_port);

         fields.port=m;
         delete[] m;
         fields.password="";
         p_passworddialog(&fields,server_needs());
         if (!fields.cancel)
         {
	   if (server_needs()&needs_host)
	       	set_host(fields.host.c_str());
	   if (server_needs()&needs_username)
  		set_user(fields.user.c_str());
	   if (server_needs()&needs_password)
    		set_password(fields.password.c_str());
	   if (server_needs()&needs_sqldelimiter)
    		p_sqldelimiter=fields.sql_delimiter.c_str();
	   if (server_needs()&needs_port)
    		p_tcp_port=atoi(fields.port.c_str());
    		//cout <<"TCP PORT HK_CONNECTION: "<<tcp_port()<<endl;
	   if (fields.save) save_configuration();
         }
         return fields.cancel;
       }
      else
      {
	cerr <<hk_translate("Please give me the following information:")<<endl;
	cerr <<"========================================="<<endl;

	if (server_needs()&needs_host)
   	{
	cerr <<hk_translate("Host: ");
	cin  >> fields.host;
   	}
	if (server_needs()&needs_username)
   	{
	cerr <<hk_translate("User: ");
	cin >> fields.user;
  	set_user(fields.user.c_str());
   	}
	if (server_needs()&needs_password)
   	{
	cerr <<hk_translate("Password (will not be displayed): ");
	struct termios initialsettings,newsettings ;
 	tcgetattr(fileno(stdin),&initialsettings);
 	newsettings=initialsettings;
 	newsettings.c_lflag &=~ECHO;
 	tcsetattr(fileno(stdin),TCSAFLUSH,&newsettings);
	cin >> fields.password;
 	tcsetattr(fileno(stdin),TCSAFLUSH,&initialsettings);
 	cerr<<endl;
    	set_password(fields.password.c_str());
   	}

	if (server_needs()&needs_port)
   	{
	cerr <<hk_translate("TCP Port: ");
	cin >> fields.port;
    	set_tcp_port(atoi(fields.port.c_str()));
   	}

	if (server_needs()&needs_sqldelimiter)
   	{
	cerr <<hk_translate("SQL Delimiter: ");
	cin >> fields.sql_delimiter;
    	}

   if (server_needs()&needs_host)
       	set_host(fields.host.c_str());
   if (server_needs()&needs_username)
	set_user(fields.user.c_str());
   if (server_needs()&needs_password)
	set_password(fields.password.c_str());
   if (server_needs()&needs_sqldelimiter)
	p_sqldelimiter=fields.sql_delimiter.c_str();
   if (server_needs()&needs_port)
	p_tcp_port=atoi(fields.port.c_str());
  	   return false;//not cancel

     }

   return false;
}

bool hk_connection::disconnect()
{
 	return driver_specific_disconnect();
}


const vector<hk_string>* hk_connection::dblist(void)
{
 return driver_specific_dblist();

}


void hk_connection::db_remove(hk_database* d)
{
	if (p_database==d) p_database=NULL;
}


hk_database* hk_connection::new_database(const hk_string& name)
{
#ifdef HK_DEBUG
hkdebug("hk_connection::new_database");
#endif
    hk_database* db;
    if (p_database!=NULL)
    {
     delete p_database;
    }
    db=driver_specific_new_database();
//    hk_string n;
    p_database=db;
    if (name.size()>0)db->set_name(name);

    hk_string p=p_databasepath+"/"; p+=name;
    mkdir (p.c_str(),S_IRUSR|S_IWUSR|S_IXUSR);


    return db;
}

bool hk_connection::is_connected(void)
{
  	return p_connected;
}

void hk_connection::inform_databases_connect()
{
 if (p_database!=NULL) p_database->connection_connected();
}

bool hk_connection::create_database(const hk_string& dbase)
{
#ifdef HK_DEBUG
hkdebug("hk_connection::create_database");
#endif
  if (p_database==NULL) new_database();
  if (p_database==NULL||!is_connected()) return false;
  hk_actionquery* q=p_database->new_actionquery();
  hk_string sql="CREATE DATABASE ";
  sql+=dbase;
  sql+=p_sqldelimiter;
  bool result=false;
  if (q!=NULL)
  {
     q->set_sql(sql.c_str(),sql.size());
     result=q->execute();
     delete  q;
  }
  return result;
}

bool hk_connection::delete_database(const hk_string& dbase)
{
  if (p_database==NULL) new_database();
  if (p_database==NULL||!is_connected()) return false;
  hk_actionquery* q=p_database->new_actionquery();
  hk_string sql="DROP DATABASE ";
  sql+=dbase;
  sql+=p_sqldelimiter;
  bool result=false;
  hk_string warning=hk_translate("Delete the database \"%DBNAME%\"?");
  warning=replace_all("%DBNAME%",warning,dbase);
  if(show_yesnodialog(warning,true))
    {

      if (q!=NULL)
         {
           q->set_sql(sql.c_str(),sql.size());
           result=q->execute();
           delete  q;
         }
    } else return false;
    if (result)
    {
      hk_string n=dbase;
      if (p_database!=NULL) if (p_database->name()==n)
                               {
			         delete  p_database;
				 p_database=NULL;
			       }
    }
    else
    show_warningmessage(hk_translate("Database could not be deleted!"));
  return result;

}

void hk_connection::savedata(ofstream& s)
{
        set_tagvalue(s,"HOST",p_host);
        set_tagvalue(s,"USER",p_user);
        set_tagvalue(s,"TCP-PORT",(long unsigned int)p_tcp_port);
        //set_tagvalue(s,"SQLDELIMITER",p_sqldelimiter);


}

void hk_connection::save_configuration(void)
{
  mkdir (p_classespath.c_str(),S_IRUSR|S_IWUSR|S_IXUSR);
  hk_string p_filename =p_classespath+"/";
  p_filename+="driver.conf";
  ofstream* stream= new ofstream(p_filename.c_str());
  if (stream==NULL)return;
  savedata(*stream);
}

void  hk_connection::loaddata(const hk_string& definition)
{
#ifdef HK_DEBUG
hkdebug("hk_connection::loaddata");
#endif
     hk_string buffer;
     long unsigned int intbuffer;
        if (get_tagvalue(definition,"HOST",buffer)) set_host(buffer) ;//cout <<"host: "<<p_host<<endl;
        if (get_tagvalue(definition,"USER",buffer)) set_user(buffer);//;cout <<"user: "<<p_user<<endl;
        if (get_tagvalue(definition,"TCP-PORT",intbuffer))set_tcp_port(intbuffer);//;cout <<"tcp: "<<p_tcp_port<<endl;
        //if (get_tagvalue(definition,"SQLDELIMITER"),buffer)set_sqldelimiter(buffer);
}

void hk_connection::load_configuration(void)
{
#ifdef HK_DEBUG
hkdebug("hk_connection::load_configuration");
#endif
//cout <<"load_configuration"<<endl;
   hk_string n=p_classespath+"/";
   n+="driver.conf";
   long size;
   FILE *in;  //cout <<"Lade Datei:"<<n<<endl;
   in=fopen(n.c_str(),"r");
   if (in)
   {
      fseek(in,0,SEEK_END);
      size=ftell(in);
      fseek(in,0,SEEK_SET);
      char* p_new_data=new char[size];
      fread(p_new_data,size,1,in);
      loaddata(p_new_data);
      delete[] p_new_data;
   }

}


void hk_connection::set_classespath(hk_string& p)
{
	p_classespath=p;
	p_databasepath=p;
	load_configuration();
}
