// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.9 $
#include "hk_database.h"
#include "hk_data.h"
#include "hk_dbvisible.h"
#include "hk_presentation.h"
#include "hk_connection.h"
#include "hk_datasource.h"
#include "hk_actionquery.h"

#include <sys/stat.h>
#include <iostream.h>
#include <dirent.h>
#include <unistd.h>

hk_database::hk_database(hk_connection* c)
{
  p_tablelist = new vector<hk_string>;
  p_connection=c;

}


hk_database::~hk_database()
{
#ifdef HK_DEBUG
hkdebug("hk_database::destructor");
#endif
   inform_datasources_before_closing();
   clear_presentationlist();
   p_connection->db_remove(this);
   list<hk_data*>::iterator it;
   for (it=p_hkdsourcelist.begin();it!=p_hkdsourcelist.end();)
   {
     hk_data* dt=*it ;
     it++;
     delete dt;
   }
   p_hkdsourcelist.erase(p_hkdsourcelist.begin(),p_hkdsourcelist.end());
   if (p_tablelist!=NULL) delete p_tablelist;

}


void hk_database::set_name(const hk_string& n)
{
  if (n==p_dbname)return;
  p_dbname=n;
  select_db();
}

hk_string hk_database::name(void)
{

	return p_dbname;
}

vector<hk_string>* hk_database::tablelist(void)
{
 return driver_specific_tablelist();
}

bool hk_database::table_exists(const hk_string& tablename)
{
 bool res=false;
 const vector<hk_string>* liste=tablelist();
 vector<hk_string>::iterator it;
 it=(hk_string*) liste->begin();
 while (it!=liste->end())
 {
  if ((*it)==tablename)return true;

  it++;
 }

 return res;
}

hk_datasource* hk_database::new_table(const hk_string& name,hk_presentation* p)
{
   hk_datasource* tb;
   tb=driver_specific_new_table(p);
   if (tb!=NULL)
   {
      p_hkdsourcelist.insert(p_hkdsourcelist.end(),tb);
      if (p_connection->server_needs()&needs_sqldelimiter)
         tb->set_sqldelimiter(p_connection->sqldelimiter());

   }

   if (name.size()>0)tb->set_name(name);
   return tb;
}

hk_datasource* hk_database::new_resultquery(hk_presentation* p)
{
   hk_datasource* tb;
   tb=driver_specific_new_resultquery(p);
   if (tb!=NULL)
   {
     p_hkdsourcelist.insert(p_hkdsourcelist.end(),tb);
     if (p_connection->server_needs()&needs_sqldelimiter)
         tb->set_sqldelimiter(p_connection->sqldelimiter());
   }
   return tb;
}

hk_actionquery* hk_database::new_actionquery(void)
{
   hk_actionquery* tb;
   tb=driver_specific_new_actionquery();
   if (tb!=NULL) p_hkdsourcelist.insert(p_hkdsourcelist.end(),tb);
   return tb;
}


void hk_database::ds_remove(hk_data* ds)
{
#ifdef HK_DEBUG
hkdebug("hk_database::ds_remove");
#endif

 	p_hkdsourcelist.remove(ds);

}

bool hk_database::select_db(void)
{
 inform_datasources_before_closing();
 if (p_connection->is_connected())
 {
 	bool erg= driver_specific_select_db();
 	if (erg)
	 {
	  p_databasepath=p_connection->databasepath();
	  p_databasepath+="/";
	  p_databasepath+=name();
          mkdir (p_databasepath.c_str(),S_IRUSR|S_IWUSR|S_IXUSR);

	 }
	else
	  show_warningmessage(hk_translate("No such Database!"));
 	return erg;
 }
 else
 {
 return false;
 }


}


void hk_database::connection_connected(void)
{
 select_db();

}
bool hk_database::delete_table(const hk_string& table)
{
  if (!p_connection->is_connected()) return false;
  hk_actionquery* q=new_actionquery();
  hk_string sql="DROP TABLE ";
  sql+=table+p_sqldelimiter;
  bool result=false;
  hk_string warning=hk_translate("Delete the table \"%TBLNAME%\"?");
  warning=replace_all("%TBLNAME%",warning,table);
  if(show_yesnodialog(warning,true))
    {

      if (q!=NULL)
         {
           q->set_sql(sql.c_str(),sql.size());
           result=q->execute();
           delete  q;
         }
    } else return false;
    if (result)
    {
   	list<hk_data*>::iterator it ;
	it=p_hkdsourcelist.begin();
	while (it!=p_hkdsourcelist.end())
     	  {
       	    hk_data* vi=*it;
	    hk_string t=table;
       	    if (vi->type()==hk_data::ds_table && vi->name()==t)vi->before_source_vanishes();
       	    it++;
     	  }

    }
    else show_warningmessage(hk_translate("Table could not be deleted!"));
  return result;


}

void hk_database::inform_datasources_before_closing(void)
{
#ifdef HK_DEBUG
hkdebug("hk_database::inform_datasources_before_closing");
#endif
   	list<hk_data*>::iterator it ;
	it=p_hkdsourcelist.begin();
	while (it!=p_hkdsourcelist.end())
     	  {
       	    hk_data* vi=(*it);
    	    if (vi!=NULL)vi->before_source_vanishes();

       it=find(p_hkdsourcelist.begin(),p_hkdsourcelist.end(),vi);
       if (it==p_hkdsourcelist.end()) it=p_hkdsourcelist.begin();
       else it++;
     	  }

}
void hk_database::save(const hk_string& statement,const hk_string& name,filetype type)
{
#ifdef HK_DEBUG
hkdebug("hk_database::save");
#endif
   hk_string n=p_databasepath+"/";
   n+=name+fileendings(type);
//   cout <<n<<endl;
   FILE* out=fopen(n.c_str(),"w");

  int r=fwrite(statement.c_str(),statement.size(),1,out);
   fclose(out);
//   if (r<(signed)res->length)return false;
//return true;

}

ofstream* hk_database::savestream(const hk_string&name,filetype type,bool ask_before_overwrite)
{
   hk_string filename=p_databasepath+"/";
   filename+=name+fileendings(type);
   hk_string dialog;
   if (type==ft_query)dialog=hk_translate("Query already exists. Overwrite it?");
   else if (type==ft_form)dialog=hk_translate("Form already exists. Overwrite it?");
   else if (type==ft_report)dialog=hk_translate("Report already exists. Overwrite it?");
   else dialog=hk_translate("File already exists. Overwrite it?");
  if (ask_before_overwrite)
  {
    ifstream ifs(filename.c_str(),ios::in);
    if (ifs)
      {
         //file already exists, overwrite it?
      	if(!show_yesnodialog(dialog,true))
           return NULL;
     }
  }

  ofstream* stream= new ofstream(filename.c_str());
  if (stream!=NULL)  inform_datasources_filelist_changes(type);
  return stream;

}


hk_string hk_database::load(const hk_string& name,filetype type)
{
#ifdef HK_DEBUG
hkdebug("hk_database::load");
#endif

   hk_string n=p_databasepath+"/";
   n+=name+fileendings(type);
   long size;
   FILE *in; // cout <<"Lade Datei:"<<n<<endl;
   in=fopen(n.c_str(),"r");
   if (in==NULL)return "";
   fseek(in,0,SEEK_END);
   size=ftell(in);
   fseek(in,0,SEEK_SET);
  char* p_new_data=new char[size];
  fread(p_new_data,size,1,in);
//  cout << p_new_data <<"&"<<endl;
  return p_new_data;
}


hk_string hk_database::fileendings(filetype e)
{
  switch (e)
  {

   case ft_query: return ".hk_query";
   case ft_form: return ".hk_form";
   case ft_report: return ".hk_report";
  }

}

vector<hk_string>* hk_database::querylist(void)
{
   return filelist(ft_query);
}
vector<hk_string>* hk_database::formlist(void)
{
   return filelist(ft_form);
}
vector<hk_string>* hk_database::reportlist(void)
{
   return filelist(ft_report);
}
/*     DIR *dp;
     struct dirent *entry;
     hk_string datei;

 p_querylist.erase(p_querylist.begin(),p_querylist.end());
   if ((dp=opendir(p_databasepath.c_str()))!=NULL)
   {

     while ((entry = readdir(dp))!=NULL)
     {
       datei=entry->d_name;
       size_t p =datei.find(fileendings(ft_query));
       if (p<datei.size())
       {
          datei.replace(p,datei.size()-p,"");
          p_querylist.insert(p_querylist.end(),datei);
       }
     }
   }




 return &p_querylist;
*/
//}



vector<hk_string>* hk_database::filelist(filetype type)
{
     DIR *dp;
     struct dirent *entry;
     hk_string datei;

 p_filelist.erase(p_filelist.begin(),p_filelist.end());
   if ((dp=opendir(p_databasepath.c_str()))!=NULL)
   {

     while ((entry = readdir(dp))!=NULL)
     {
       datei=entry->d_name;
       size_t p =datei.find(fileendings(type));
       if (p<datei.size())
       {
          datei.replace(p,datei.size()-p,"");
          p_filelist.insert(p_filelist.end(),datei);
       }
     }
   }
  sort(p_filelist.begin(),p_filelist.end());
 return &p_filelist;
}


bool  hk_database::delete_file(const hk_string& name,filetype type)
{
   hk_string filename=p_databasepath+"/";
   filename+=name+fileendings(type);
   hk_string message;
   if (type==ft_query)
      message=hk_translate("Delete the query:%FNAME%?");
   else
   if (type==ft_report)
      message=hk_translate("Delete the report:%FNAME%?");
   else
   if (type==ft_form)
      message=hk_translate("Delete the form:%FNAME%?");
   else
      message=hk_translate("Delete the file:%FNAME%");
   message=replace_all("%FNAME%",message,name);
   if(!show_yesnodialog(message,true))
           return false;
   int r=unlink(filename.c_str());
   if (r==0)
   {
    inform_datasources_filelist_changes(type);
    return true;
   }
   return false;
}
void hk_database::inform_datasources_filelist_changes(filetype type)
{
#ifdef HK_DEBUG
hkdebug("hk_database::inform_datasources_filelist_changes");
#endif
   	list<hk_data*>::iterator it ;
	it=p_hkdsourcelist.begin();
	while (it!=p_hkdsourcelist.end())
     	  {
	    hk_data* vi=*it;
       	    it++;
       	    vi->filelist_changes(type);
     	  }


}


hk_datasource* 	hk_database::load_datasource(const hk_string& name,bool query,hk_presentation* p)
{
#ifdef HK_DEBUG
hkdebug("hk_database::load_datasource");
#endif
if (query)
	{
   	 hk_datasource* d=NULL;
	 hk_string r=load(name,ft_query);
	 if (r.size()>0)
	 	{
		 d=new_resultquery(p);
		}
	 if (d!=NULL) d->loaddata(r);
	return d;
	}
else
 return new_table(name,p);

}

hk_datasource* hk_database::driver_specific_new_table(hk_presentation* p)
{
   return NULL;
}

hk_datasource* hk_database::driver_specific_new_resultquery(hk_presentation* p)
{
   return NULL;
}

void hk_database::dbvisible_add(hk_dbvisible* v)
{
#ifdef HK_DEBUG
hkdebug("hk_database::visible_add");
#endif
	p_visibles.insert(p_visibles.end(),v);
}


void hk_database::dbvisible_remove(hk_dbvisible* v)
{
#ifdef HK_DEBUG
hkdebug("hk_database::visible_remove");
#endif
	p_visibles.remove(v);
}

void hk_database::clear_visiblelist(void)
{
#ifdef HK_DEBUG
hkdebug("hk_database::clear_visiblelist");
#endif
        list<hk_dbvisible*>::iterator it;
        it=p_visibles.begin();
	while ( it!=p_visibles.end() )
     	{
       		hk_dbvisible* dv=*it;
                 ++it;
		dv->database_delete();
		p_visibles.remove(dv);
     	}


}


void hk_database::before_source_vanishes(void)
{
  #ifdef HK_DEBUG
 hkdebug("hk_database::before_source_vanishes");
 #endif
   list<hk_dbvisible*>::iterator it =p_visibles.begin();
   while(it!=p_visibles.end())
     {
       hk_dbvisible* vi=(*it);
       vi->before_source_vanishes();
       it=find(p_visibles.begin(),p_visibles.end(),vi);
       if (it==p_visibles.end()) it=p_visibles.begin();
       else it++;
     }
#ifdef HK_DEBUG
 hkdebug("ENDE database::before_source_vanishes");
#endif
}


void hk_database::presentation_add(hk_presentation* p)
{
#ifdef HK_DEBUG
hkdebug("hk_database::presentation_add");
#endif
	p_presentations.insert(p_presentations.end(),p);
}


void hk_database::presentation_remove(hk_presentation* p)
{
#ifdef HK_DEBUG
hkdebug("hk_database::presentation_remove");
#endif
	p_presentations.remove(p);
}


void hk_database::clear_presentationlist(void)
{
#ifdef HK_DEBUG
hkdebug("hk_database::clear_presentationlist");
#endif
        list<hk_presentation*>::iterator it;
        it=p_presentations.begin();
	while ( it!=p_presentations.end() )
     	{
       		hk_presentation* dv=*it;
		#ifdef HK_DEBUG
		hkdebug("dv->classname: ");
		hkdebug(dv->hkclassname());
		#endif
                 ++it;
		dv->database_delete();
		p_presentations.remove(dv);
     	}


}

hk_presentation* hk_database::existing_presentation(const hk_string& name, hk_presentation::enum_presentationtype t)
{
#ifdef HK_DEBUG
hkdebug("hk_database::existing_presentation");
#endif
        list<hk_presentation*>::iterator it;
        it=p_presentations.begin();
	while ( it!=p_presentations.end() )
     	{
       		hk_presentation* dv=*it;
                 ++it;
		if (dv->name()==name &&dv->presentationtype()== t)
		{
		return dv;
		}
     	}
  return NULL;
}
