// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.29 $
#include <hk_datasource.h>
#include <hk_database.h>
#include <hk_dsvisible.h>
#include <algorithm>
#include <list>
#include <stdlib.h>
#include <hk_connection.h>
#include <fstream.h>
#include <hk_presentation.h>
#include <hk_actionquery.h>


hk_datasource::hk_datasource(hk_database* db,hk_presentation* p):hk_data()
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::hk_datasource");
#endif
 p_true="YES";
 p_false="NO";
 p_database=db;
 p_presentation=p;
 p_enabled=false;
 p_counter=0;
 p_length=0;
 set_has_not_changed();
 p_primary_key_used=false;
 p_mode=mode_disabled;
 p_ignore_changed_data=false;
 p_depending_on_datasource=NULL;
 p_parsed_sql=NULL;
 p_readonly=false;
 p_filter="";
 p_automatic_data_update=true;
 p_actionquery=NULL;
 p_columns=NULL;
 p_oldcolumns=NULL;
 p_enable_phase=false;
 p_tablesql_set=false;
 p_rawsql=false;
 p_dscounter=-1;
// wanna_debug(true);
 p_depending_on_presentationdatasource=-1;
 internal_dsrefresh=false;
 p_internal=NULL;
 p_depending_on_datasource_react_on_changed_data=false;
 p_dependingmode=depending_standard;
}



hk_datasource::~hk_datasource()
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::~hk_datasource");
#endif

  	clear_visiblelist();
  	clear_columnlist();
	clear_modecolumnlists();
  	list<hk_datasource*>::iterator dep_it;
  	dep_it=p_dependinglist.begin();
	while (dep_it!=p_dependinglist.end())
	{
	 	(*dep_it)->set_depending_on(NULL);
		dep_it++;
	}
    if (p_oldcolumns!=NULL)
      {
        list<hk_column*>::iterator it;
        it=p_oldcolumns->begin();
	while ( it!=p_oldcolumns->end() )
	  {
                 hk_column* x=*it;
		 ++it;
		 delete x;
	  }
      }
  p_database->ds_remove(this);
  if (p_presentation!=NULL) p_presentation->remove_datasource(p_dscounter);
  if (p_depending_on_datasource!=NULL)
  	p_depending_on_datasource->depending_ds_remove(this);
  	p_depending_on_datasource=NULL;
 if (p_actionquery!=NULL) delete p_actionquery;
 if (p_parsed_sql!=NULL) delete p_parsed_sql;
#ifdef HK_DEBUG
hkdebug("hk_datasource::~hk_datasource ENDE");
#endif

}


void hk_datasource::set_name(const hk_string& n,bool registerchange)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::set_name");
#endif
   if (p_presentation!=NULL&& n.size()>0&&registerchange) p_presentation->set_has_changed();
   hk_data::set_name(n);
   if ( type()==ds_table)
     {
		hk_string t="SELECT * FROM ";
		t+=n+p_sql_delimiter;
		p_tablesql_set=true;
		set_sql(t);
		p_tablesql_set=false;

     }


}

hk_string hk_datasource::name(void)
{
   return hk_data::name();
}

void hk_datasource::setmode_createtable(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::setmode_createtable");
#endif
    if (type()!=ds_table || runtime_only())return;
    if (is_enabled()) disable();
    clear_columnlist();
    clear_modecolumnlists();

    p_mode=mode_createtable;
}


void hk_datasource::setmode_normal(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::setmode_normal");
#endif

   if (p_mode==mode_createtable)create_table_now();
   if (p_mode==mode_altertable) alter_table_now();
   if (!is_enabled()&&!p_enable_phase) enable();
   enum_tablemodes buffermode=p_mode;
   p_mode=mode_normal;
   if (buffermode==mode_insertrow)  inform_visible_objects_row_change();
}


void hk_datasource::setmode_insertrow(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::setmode_insertrow");
#endif
 	if (is_readonly())return;
 	switch (p_mode)
 	{
 	  case mode_normal:
 	  case mode_insertrow:store_changed_data();
 	  			break;
 	  case mode_disabled:;
 	  case mode_altertable:enable();
 	  default  :;
 	}

 	p_mode=mode_insertrow;
	if (p_depending_on_datasource!=NULL)
	{
	  list<hk_string>::iterator m=depending_master_fields.begin();
	  for (list<hk_string>::iterator k=depending_this_fields.begin();k!=depending_this_fields.end();k++)
	   {//set the depending fields with the value of the master field
	      hk_column* kcol=column_by_name((*k));
              if (kcol!=0)
		{
			hk_column* nk=p_depending_on_datasource->column_by_name((*m));
			if (nk!=0)
			kcol->set_asstring(nk->asstring());
	        }

   	m++;
            }
        set_has_not_changed();
 	}
 //    inform_visible_objects_row_change();
     inform_visible_objects_insertmode();
     inform_depending_ds_insertmode();

}



bool hk_datasource::create_table_now(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::create_table_now");
#endif
 if ((p_mode==mode_createtable)&&(p_name.size()>0))
   {
     bool res=driver_specific_create_table_now();
     if (res)
     {
	p_mode=mode_disabled;
	p_database->inform_datasources_filelist_changes(ft_table);
     }
     else show_warningmessage(hk_translate("Table could not be created."));
     return res;
   }
 else return false;
}



void hk_datasource::delete_actualrow(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::delete_actualrow");
#endif
	if (p_mode==mode_normal)
 	   {
 		 p_mode=mode_deleterow;
 		 set_has_changed();
	 	store_changed_data();

 	  }

}

hk_datasource::enum_tablemodes hk_datasource::mode(void)
{
 	return p_mode;
}


void hk_datasource::column_remove(hk_column* col)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::column_remove");
#endif
   if (p_columns!=NULL) p_columns->remove(col);
}


void hk_datasource::inform_visible_objects_row_change(void)
{
 			// Inform the visual Objects that data has changed
  #ifdef HK_DEBUG
 hkdebug("datasource::inform_visible_objects_row_change");
 #endif

   	list<hk_dsvisible*>::iterator it=p_visibles.begin() ;
	while(it!=p_visibles.end())
	  {
		hk_dsvisible* vi=*it;
       		it++;
       		vi->row_change();
     	  }


}
void hk_datasource::inform_visible_objects_before_insert_row()
{
  #ifdef HK_DEBUG
 hkdebug("datasource::inform_visible_objects_row_change");
 #endif

   	list<hk_dsvisible*>::iterator it=p_visibles.begin() ;
	while(it!=p_visibles.end())
	  {
		hk_dsvisible* vi=*it;
       		it++;
       		vi->before_insert_row();
     	  }

}

void  	hk_datasource::inform_when_table_structure_changes(void)
{
  #ifdef HK_DEBUG
 hkdebug("datasource::inform_when_table_structure_changes");
 #endif

   	list<hk_dsvisible*>::iterator it ;
	for(it=p_visibles.begin();it!=p_visibles.end();)
	  {
		hk_dsvisible* vi=*it;
       		it++;
       		vi->table_structure_changes();
     	  }

}

void  	hk_datasource::inform_when_indexlist_changes(void)
{
  #ifdef HK_DEBUG
 hkdebug("datasource::inform_when_table_structure_changes");
 #endif

   	list<hk_dsvisible*>::iterator it ;
	for(it=p_visibles.begin();it!=p_visibles.end();)
	  {
		hk_dsvisible* vi=*it;
       		it++;
       		vi->list_changes(ft_index);
     	  }

}


void hk_datasource::filelist_changes(filetype t)
{
  #ifdef HK_DEBUG
 hkdebug("datasource::filelist_changes");
 #endif
	list<hk_dsvisible*>::iterator it ;
	for(it=p_visibles.begin();it!=p_visibles.end();)
     	  {
       		hk_dsvisible* vi=*it;
       		vi->list_changes(t);
       		it++;
     	  }


}

void hk_datasource::before_source_vanishes(void)
{
  #ifdef HK_DEBUG
 hkdebug("datasource::before_source_vanishes");
 #endif
   store_changed_data();
   list<hk_dsvisible*>::iterator it =p_visibles.begin();
   while(it!=p_visibles.end())
     {
       hk_dsvisible* vi=(*it);
       vi->before_source_vanishes();
       it=find(p_visibles.begin(),p_visibles.end(),vi);
       if (it==p_visibles.end()) it=p_visibles.begin();
       else it++;
     }
     if (p_presentation!=NULL) p_presentation->remove_datasource(this);
     p_presentation=NULL;
   disable();
#ifdef HK_DEBUG
 hkdebug("ENDE datasource::before_source_vanishes");
#endif
}

void hk_datasource::inform_visible_objects_insertmode(void)
{
 			// Inform the visual Objects that data has changed
  #ifdef HK_DEBUG
 hkdebug("datasource::inform_visible_objects_insertmode");
 #endif

 		   	list<hk_dsvisible*>::iterator it ;
   			for(it=p_visibles.begin();it!=p_visibles.end();)
     			  {
       				hk_dsvisible* vi=*it;
       				it++;
       				vi->insert_mode();
     			  }


}

void hk_datasource::inform_visible_objects_ds_disable(void)
{
 #ifdef HK_DEBUG
 hkdebug("datasource::inform_visible_objects_ds_disable");
 #endif
 		   	list<hk_dsvisible*>::iterator it ;
   			for(it=p_visibles.begin();it!=p_visibles.end();)
     			  {
       				hk_dsvisible* vi=*it;
       				it++;
       				vi->datasource_disable();
     			  }
}

void hk_datasource::inform_visible_objects_ds_enable(void)
{
 #ifdef HK_DEBUG
 hkdebug("datasource::inform_visible_objects_ds_enable");
 #endif
 		   	list<hk_dsvisible*>::iterator it ;
   			for(it=p_visibles.begin();it!=p_visibles.end();)
     			  {
       				hk_dsvisible* vi=*it;
       				it++;
       				vi->datasource_enable();
     			  }
}


bool hk_datasource::goto_row(unsigned long r)
{
 #ifdef HK_DEBUG
 hkdebug("datasource::gotorow");
 #endif

    	inform_before_row_change();
    	if ((p_automatic_data_update)&&(p_has_changed)) store_changed_data();
    	if (p_mode==mode_insertrow)setmode_normal();
    	if (driver_specific_goto_row(r))
  	{
		p_counter=r;
                inform_depending_ds_goto_row();
                inform_visible_objects_row_change();
    		return true;
  	}
     	inform_depending_ds_goto_row();
     	inform_visible_objects_row_change();

  	return false;

}

bool hk_datasource::goto_random(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::goto_random");
#endif
  goto_row(hk_random(0,max_rows()-1));
 return true;
}



bool hk_datasource::goto_first()
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::goto_first");
#endif
 	return goto_row(0);
}



bool hk_datasource::goto_next()
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::goto_next");
#endif
  if (p_counter<(max_rows()))
  	return goto_row(p_counter+1);
  else 	return false;
}



bool hk_datasource::goto_previous()
{
    #ifdef HK_DEBUG
   hkdebug("datasource::goto_previous");
   #endif

 if (p_counter>0)
   return goto_row(p_counter-1);
 else return false;
}



bool hk_datasource::goto_last()
{
 	return goto_row(max_rows()-1);
}


unsigned long hk_datasource::max_rows(void)
{

 	return driver_specific_max_rows();
}



bool hk_datasource::set_sql(const hk_string& s,bool rawsql,bool registerchange=true)
{

    #ifdef HK_DEBUG
   hkdebug("datasource::sql");
   #endif
if ((type()==ds_table)&&(!p_tablesql_set)) return false;
   if ((p_presentation!=NULL&&type()==ds_query) &&registerchange) p_presentation->set_has_changed();
   p_rawsql=rawsql;
 	p_sql=s;
 	p_original_sql=s;
        parse_sql();
	if (!p_rawsql)
	{
 	   create_new_sql_statement();
	}
	else
	{
  	p_sql=replace_all("%TRUE%",p_sql,p_true);
  	p_sql=replace_all("%FALSE%",p_sql,p_false);

	}
	return true;
}

hk_string hk_datasource::sql(void)
{
 	return p_original_sql;
}

bool hk_datasource::is_readonly(void)
{
	if (type()==ds_query) return true;
	else return p_readonly;
}

void hk_datasource::set_readonly(bool r)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::set_readonly");
#endif
  	if (type()==ds_table)
  	{
  	 p_readonly=r;
  	}
}

bool hk_datasource::enable(void)
{
     #ifdef HK_DEBUG
   hkdebug("datasource::enable");
   #endif
   if (p_presentation!=NULL)
   {
     if ( p_depending_on_presentationdatasource>-1)
     if (p_depending_on_datasource==NULL)
         set_depending_on_presentationdatasource(p_depending_on_presentationdatasource,p_depending_on_datasource_react_on_changed_data,p_dependingmode);
   }

   if (!p_database->p_connection->is_connected())p_database->p_connection->connect();
   if (p_depending_on_datasource!=NULL)
   	if (!p_depending_on_datasource->is_enabled())return false;
  if (!p_enabled)
  {
   if (p_columns!=NULL) clear_columnlist();
   if (p_depending_on_datasource!=NULL) create_new_sql_statement();
   if (driver_specific_enable())
     {
       p_enabled=true;
//       goto_first();
	p_counter=0;
      inform_visible_objects_new_columns_created();
       setmode_normal();
       inform_depending_ds_enable();
        inform_visible_objects_ds_enable();
       if (max_rows()==0) setmode_insertrow();
       return true;
      }
   else return false;
  };
  return false;
}



bool hk_datasource::disable(void)
{
#ifdef HK_DEBUG
   hkdebug("datasource::disable");
#endif
     	inform_before_disable();
	if (p_automatic_data_update) store_changed_data();
  	p_enabled=false;
  	p_mode=mode_disabled;
        bool erg=driver_specific_disable();
        inform_visible_objects_ds_disable();
        inform_depending_ds_disable();

        return erg;
}


bool hk_datasource::is_enabled(void)
{
	return p_enabled;

}



unsigned long hk_datasource::row_position(void)
{
	return p_counter;
}


 list<hk_column*>* hk_datasource::columns(void)
{
 	return driver_specific_columns();
}


void hk_datasource::clear_columnlist(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::clear_columnlist");
#endif
inform_visible_objects_before_columns_deleted();
      if (p_oldcolumns!=NULL)
      {
        list<hk_column*>::iterator it;
        it=p_oldcolumns->begin();
	while ( it!=p_oldcolumns->end() )
	  {
                 hk_column* x=*it;
		 ++it;
		 delete x;
	  }
        delete p_oldcolumns;
        }
        p_oldcolumns=p_columns;
        p_columns=0;
        p_primary_key_used=false;
}

void	hk_datasource::clear_modecolumnlists(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::clear_modecolumnlists");
#endif
        list<hk_column*>::iterator it;
        it=p_newcolumns.begin();
	while ( it!=p_newcolumns.end() )
	  {
                 hk_column* x=*it;
		 ++it;
		 delete x;
	  }
	p_newcolumns.erase(p_newcolumns.begin(),p_newcolumns.end());
        p_altercolumns.erase(p_altercolumns.begin(),p_altercolumns.end());
	p_deletecolumns.erase(p_deletecolumns.begin(),p_deletecolumns.end());

}




hk_column* hk_datasource::new_column(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::new_column");
#endif
 if ((p_mode==mode_createtable)||(p_mode==mode_altertable))
    {
      hk_column* c= driver_specific_new_column();
      if (c!=NULL)   p_newcolumns.insert(p_newcolumns.end(),c);
      return c;
    }
 else return NULL;

}


void hk_datasource::visible_add(hk_dsvisible* v)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::visible_add");
#endif
	p_visibles.insert(p_visibles.end(),v);
}


void hk_datasource::visible_remove(hk_dsvisible* v)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::visible_remove");
#endif
	p_visibles.remove(v);
}


void hk_datasource::clear_visiblelist(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::clear_visiblelist");
#endif
        list<hk_dsvisible*>::iterator it;
        it=p_visibles.begin();
	while ( it!=p_visibles.end() )
     	{
       		hk_dsvisible* dv=*it;
		#ifdef HK_DEBUG
		hkdebug("dv->classname: ");
		hkdebug(dv->hkclassname());
		#endif
                 ++it;
		dv->datasource_delete();
		p_visibles.remove(dv);
     	}


}



hk_column* hk_datasource::column_by_name(const hk_string& c)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::column_by_name(char) ",c);
#endif
	list<hk_column*>* p_l =columns();

      if (p_l!=NULL)
      {
       #ifdef HK_DEBUG
hkdebug("hk_datasource::column_by_name ","p_columns!=NULL");
#endif

        list<hk_column*>::iterator it;
        it=p_l->begin();
	while ( it!=p_l->end())
	  {
                 hk_column* x=*it;
		 ++it;
		 hk_string s=x->name();
		 if (s==c) return x;
	  }
       }
       else
       {
       #ifdef HK_DEBUG
hkdebug("hk_datasource::column_by_name ","p_columns=NULL");
#endif

       }
       #ifdef HK_DEBUG
hkdebug("hk_datasource::column_by_name ","return=NULL!!!");
#endif
              return NULL;

}


bool hk_datasource::store_changed_data(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::store_changed_data");
 	 if (p_has_changed)hkdebug("p_ds_datachanged = true"); else hkdebug("p_ds_datachanged = false");
#endif
      inform_visible_objects_before_store_changed_data();
      if ((p_ignore_changed_data) ||(p_readonly)||(!p_has_changed))
      {
        #ifdef HK_DEBUG
        hkdebug("Ignore changed Data!");
        #endif
        p_ignore_changed_data=false;
        reset_changed_data();
        if (p_mode==mode_insertrow)
           {
             p_mode=mode_normal;
             goto_row(p_counter);
           }
        p_mode=mode_normal;
        inform_depending_ds_after_store_changed_data();
	inform_visible_objects_after_store_changed_data();
	return true;
      };

      create_actual_row_where_statement();
      switch (p_mode)
      {
        case mode_normal:	update_row();
      				break;
      	case mode_deleterow:	delete_row();
      				break;
      	case mode_insertrow:	insert_row();
      				break;
      	default:		;
      }
   reset_changed_data();
   inform_depending_ds_after_store_changed_data();
   inform_visible_objects_after_store_changed_data();
   return true;
}


void hk_datasource::update_row()
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::update_row()");
#endif
      char* p_changed_data=NULL;
      char* p_it=NULL;
        unsigned long new_length=p_actual_row_where.length()+name().size()+12+p_sql_delimiter.size();

	 bool dep_bool=true;
	list<hk_datasource*>::iterator dep_it=p_dependinglist.begin();

	 while (dep_it!=p_dependinglist.end())
	 {
	  if(!(*dep_it)->depending_on_datasource_updaterow_ok()) dep_bool=false;
	  dep_it++;
	 }


	if (!dep_bool)
        {
          show_warningmessage(hk_translate("Row could not be changed due to depending datasource(s)"));
       	  p_mode=mode_normal;
          set_has_not_changed();

          return;
        }
transaction_begin();
dep_it=p_dependinglist.begin();

	 while (dep_it!=p_dependinglist.end())
	 {
	  (*dep_it)->depending_on_datasource_before_update_row();
	  dep_it++;
	 }


      if (p_columns!=NULL)
      {
        list<hk_column*>::iterator it;
               it=p_columns->begin();
	while ( it!=p_columns->end() )
        { //get Length of new data
		hk_column* x=*it;
		++it;
		if (x!=NULL)
		 if (x->has_changed())
		 {
		 #ifdef HK_DEBUG
		 hkdebug("nderungslnge ",(int)(x->transformed_changed_data()->length));
		 #endif
		     new_length+=x->transformed_changed_data()->length+6+x->name().size()+3;
		 }
        }
        p_changed_data=new char[new_length];
        *p_changed_data=0;
        p_it=p_changed_data;

        it=p_columns->begin();
	while ( it!=p_columns->end() )
	  {
                 hk_column* x=*it;
		 ++it;
		 if (x->has_changed())
		 {
		  if (*p_changed_data==0)
		  	{
		   	 strcpy(p_changed_data,"UPDATE ");
		   	 p_it+=7;

		   	 strcpy(p_it,name().c_str());
		   	 p_it+=name().size();
		   	 strcpy(p_it," SET ");
		   	 p_it+=5;
		   	 #ifdef HK_DEBUG
		   	// hkdebug(p_changed_data);
		   	 #endif
		   	 }
	 	  else
	 	     {
	 	        strcpy(p_it," , ");
	 	        p_it+=3;
	 	     }
                  strcpy(p_it,x->name().c_str());
                  p_it+=x->name().size();
                  strcpy(p_it," = \"");
                  p_it+=4;
	 	  memcpy(p_it,x->transformed_changed_data()->data,x->transformed_changed_data()->length);
	 	  p_it+=x->transformed_changed_data()->length;
	 	  strcpy(p_it,"\"");
	 	  p_it+=1;

 	         }
	  }
       }
       strcpy(p_it,p_actual_row_where.c_str());
       p_it+=p_actual_row_where.size();
       strcpy(p_it,p_sql_delimiter.c_str());
#ifdef HK_DEBUG
 hkdebug("new_length ",(int)new_length);
//hkdebug("Change hk_string ",p_changed_data);
#endif 
   set_has_not_changed();
   if (p_actionquery!=NULL)
   {
     p_actionquery->set_sql(p_changed_data,new_length);
     if (p_actionquery->execute())
        {
          #ifdef HK_DEBUG
          hkdebug("Anpassung hat funktioniert");
          #endif
          driver_specific_update_data();
	  transaction_commit();
        }
        else
        {
          #ifdef HK_DEBUG
          hkdebug("FEHLER keine Datenspeicherung");
          #endif
	  transaction_rollback();
          show_warningmessage(hk_translate("Row could NOT changed!"));
        }
   }
   delete[] p_changed_data;
dep_it=p_dependinglist.begin();

	 while (dep_it!=p_dependinglist.end())
	 {
	  (*dep_it)->depending_on_datasource_after_update_row();
	  dep_it++;
	 }

}


void hk_datasource::insert_row()
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::insert_row()");
#endif
int new_length=22+name().size()+columns()->size()*5+p_sql_delimiter.size();
char* p_it;
char* p_changed_data;
      inform_visible_objects_before_insert_row();
      if (p_columns!=NULL)
      {
        list<hk_column*>::iterator it;
               it=p_columns->begin();
	while ( it!=p_columns->end() )
        { //get Length of new data
		hk_column* x=*it;
		++it;
		if (x!=NULL)
		 if (x->has_changed())
		     new_length+=x->transformed_changed_data()->length;
        }
	 #ifdef HK_DEBUG
	 hkdebug("nderungslnge Gesamt",(int)new_length);
	 #endif


	//create INSERT SQL statement
        p_changed_data=new char[new_length];
        *p_changed_data=0;
        p_it=p_changed_data;

        it=p_columns->begin();
	while ( it!=p_columns->end() )
	  {
                 hk_column* x=*it;
		 ++it;

		  if (*p_changed_data==0)
		  	{
		   	 strcpy(p_changed_data,"INSERT INTO ");
		   	 p_it+=12;

		   	 strcpy(p_it,name().c_str());
		   	 p_it+=name().size();
		   	 strcpy(p_it," VALUES (");
		   	 p_it+=9;
		   	 }
	 	  else
	 	     {
	 	        strcpy(p_it," , ");
	 	        p_it+=3;
	 	     }


                  strcpy(p_it,"\"");
                  p_it+=1;

	//write data

	 	  if((x->transformed_changed_data()->data!=NULL))
	 	  {
	 	  	memcpy(p_it,x->transformed_changed_data()->data,x->transformed_changed_data()->length);
	 	  	p_it+=x->transformed_changed_data()->length;
	 	  }

                  strcpy(p_it,"\"");
                  p_it+=1;


          }
          strcpy(p_it,")");
          p_it+=1;
          strcpy(p_it,p_sql_delimiter.c_str());
		   	 #ifdef HK_DEBUG
		   	 hkdebug(p_changed_data);
		   	 #endif
   if (p_actionquery!=NULL)
   {
     p_actionquery->set_sql(p_changed_data,new_length);
     transaction_begin();
     if (p_actionquery->execute())
        {
          #ifdef HK_DEBUG
          hkdebug("Einfgen hat funktioniert");
          #endif
          driver_specific_insert_data();
          set_has_not_changed();
	  transaction_commit();
          goto_last();
          inform_visible_objects_row_add();
        }
        else
        {
          #ifdef HK_DEBUG
          hkdebug("FEHLER keine Datenspeicherung");
          #endif
	  transaction_rollback();
          show_warningmessage(hk_translate("Row could NOT be inserted!"));
        }
   }
   set_has_not_changed();
   delete[] p_changed_data;

     }

setmode_normal();
set_has_not_changed();
}


void hk_datasource::delete_row()
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::delete_row()");
#endif

	if(!show_yesnodialog(hk_translate("Delete this record?"),true))
	{
	#ifdef HK_DEBUG
	hkdebug("dont delete");
	#endif
	 p_mode=mode_normal;
	 set_has_not_changed();
	 return;
	}

#ifdef HK_DEBUG
hkdebug("delete");
#endif
        inform_before_row_change();
	list<hk_datasource*>::iterator dep_it;
	dep_it=p_dependinglist.begin();
	bool dep_bool=true;
	if (p_dependingmode!=depending_nohandle)
	 while (dep_it!=p_dependinglist.end())
	 {
	  if(!(*dep_it)->depending_on_datasource_deleterow_ok()) dep_bool=false;
	  dep_it++;
	 }


	if (!dep_bool)
        {
          show_warningmessage(hk_translate("Row could not be deleted due to depending datasource(s)"));
       	  p_mode=mode_normal;
          set_has_not_changed();

          return;
        }
	 transaction_begin();
	 dep_it=p_dependinglist.begin();
	 bool subbool=true;
	 while (dep_it!=p_dependinglist.end())
	 {
	  if (!(*dep_it)->depending_on_datasource_before_delete_row())
	  	subbool=false;
	  dep_it++;
	 }


        hk_string str="DELETE FROM ";
        str+= name()+p_actual_row_where+p_sql_delimiter;
#ifdef HK_DEBUG
hkdebug("DELETE SQL: ",str);
#endif
     bool exec_ok=false;
     p_actionquery->set_sql(str.c_str(),str.size());
     if (subbool)

        {
	  exec_ok=p_actionquery->execute();

	  if (exec_ok)
	  {
	     #ifdef HK_DEBUG
             hkdebug("Lschen hat funktioniert");
             #endif
             driver_specific_delete_data_at(p_counter);
             inform_visible_objects_row_delete();
	     transaction_commit();
             if ((max_rows()<=p_counter)&&(p_counter>0))p_counter=max_rows()-1;
         }
	}

    if (!subbool ||!exec_ok)
        {
          #ifdef HK_DEBUG
          hkdebug("FEHLER kein Lschen");
          #endif
	  transaction_rollback();
          show_warningmessage(hk_translate("Row was NOT deleted!"));
	  cout <<"Subbool: "<< (subbool?"TRUE":"FALSE")<<endl;
	  cout <<"sql: "<<str<<endl;
        }


        set_has_not_changed();
	if (p_counter>0)
	{
	p_mode=mode_normal;
          goto_row(p_counter);
	}
	else setmode_insertrow();
}


bool hk_datasource::depending_on_datasource_deleterow_ok(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::depending_on_datasource_deleterow_ok");
#endif
    if(p_depending_on_datasource==NULL) return true;
    bool result=true;
    switch (p_dependingmode)
    {
       case depending_delete:;
       case depending_changedelete:;
       case depending_nohandle: result=true;
       				break;
       case depending_standard: if  (max_rows()>0||type()!=ds_table) result=false;
       				break;
       default: result=true;
    }
  return result;
}

bool hk_datasource::depending_on_datasource_before_delete_row(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::depending_on_datasource_before_delete_row");
#endif
  if (type()!=ds_table) return false;
  bool result=false;
    switch (p_dependingmode)
    {
       case depending_delete:;
       case depending_changedelete:
  					{
    					hk_actionquery* a = p_database->new_actionquery();
    					if (a==NULL) return false;
    					hk_string query = "DELETE FROM ";
    					query+=name()+" ";
    					query+=whole_datasource_where_statement()+p_sql_delimiter;
    					a->set_sql(query.c_str(),query.size());
    					result=a->execute();
    					delete a;
					break;
  					}
       case depending_nohandle: result=true;
       				break;
       case depending_standard: if  (max_rows()>0||type()!=ds_table) result=false;
       				break;
       default: result=true;
    }
  return result;
}

void	hk_datasource::set_depending_on_presentationdatasource(long d,bool react_on_data_changes, enum_dependingmodes mode ,bool registerchange)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::set_depending_on_presentationdatasource");
#endif
      if (p_presentation==NULL)return;
      if (d==p_dscounter&& p_dscounter!=-1)
          {

	    show_warningmessage(hk_translate("Error: Master and depending datasource are the same!"));
	    return;
	  }
      if (registerchange) p_presentation->set_has_changed();

      p_depending_on_presentationdatasource=d;
      internal_dsrefresh=true;
      hk_datasource* ds=p_presentation->get_datasource(d);
  	if (ds!=NULL)
   	 {
    	   set_depending_on(ds,react_on_data_changes,mode);
   	 }

      internal_dsrefresh=false;

}

long    hk_datasource::depending_on_presentationdatasource(void)
{
  return p_depending_on_presentationdatasource;
}

void hk_datasource::set_depending_on(hk_datasource* d,bool react_on_data_changes,enum_dependingmodes mode)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::set_depending_on");
#endif
  if (p_presentation!=NULL &&d!=NULL)if (!internal_dsrefresh)return;
      if (d==this)
          {

	    show_warningmessage(hk_translate("Error: Master and depending datasource are the same!"));
	    return;
	  }
  p_depending_on_datasource_react_on_changed_data=react_on_data_changes;
  p_dependingmode=mode;
	if (p_depending_on_datasource!=NULL)
 	{
 	   p_depending_on_datasource->depending_ds_remove(d);
 	   p_depending_on_datasource=NULL;

	}
	if (d!=NULL)
	{
	   d->depending_ds_add(this);
	   p_depending_on_datasource=d;
	   if (p_depending_on_datasource->is_enabled()) enable();
	   else disable();
	}
}

void hk_datasource::clear_depending_fields(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::clear_depending_fields");
#endif
	depending_this_fields.erase(depending_this_fields.begin(),depending_this_fields.end());
	depending_master_fields.erase(depending_master_fields.begin(),depending_master_fields.end());

}

hk_datasource* hk_datasource::depending_on(void)
{
 if (p_depending_on_presentationdatasource>-1&&p_depending_on_datasource==NULL&&p_presentation!=NULL)
     p_depending_on_datasource =p_presentation->get_datasource(p_depending_on_presentationdatasource);
 return p_depending_on_datasource;
}


void hk_datasource::depending_ds_add(hk_datasource* d)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::depending_ds_add");
#endif
	if (d!=NULL) p_dependinglist.insert(p_dependinglist.end(),d);

}


void hk_datasource::depending_ds_remove(hk_datasource* d)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::depending_ds_remove");
#endif
	if (d!=NULL) p_dependinglist.remove(d);

}


void hk_datasource::inform_depending_ds_goto_row(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_depending_ds_goto_row");
#endif
        list<hk_datasource*>::iterator it;
        it=p_dependinglist.begin();

	while ( it!=p_dependinglist.end() )
     	{
       		hk_datasource* dv=*it;
       		++it;

       		if (dv!=NULL)
       		{  //versuchsweise gelscht am 15.07.2001
       		  //if ((p_enabled)&&(!dv->is_enabled())) dv->enable();
       		  //if ((!p_enabled)&&(dv->is_enabled())) dv->disable();
       		  dv->depending_on_datasource_row_change();
       		}
     	}

// dump_data();
}

void hk_datasource::inform_depending_ds_store_data(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_depending_ds_store_data");
#endif
        list<hk_datasource*>::iterator it;
        it=p_dependinglist.begin();

	while ( it!=p_dependinglist.end() )
     	{
       		hk_datasource* dv=*it;
       		++it;
       		if (dv->is_enabled()) dv->store_changed_data();
     	}



}

void hk_datasource::inform_depending_ds_data_has_changed(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_depending_ds_data_has_changed");
#endif
        list<hk_datasource*>::iterator it;
        it=p_dependinglist.begin();

	while ( it!=p_dependinglist.end() )
     	{
       		hk_datasource* dv=*it;
       		++it;
       		 dv->depending_on_datasource_data_has_changed();
     	}



}

void hk_datasource::inform_depending_ds_after_store_changed_data(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_depending_ds_after_store_changed_data");
#endif
        list<hk_datasource*>::iterator it;
        it=p_dependinglist.begin();

	while ( it!=p_dependinglist.end() )
     	{
       		hk_datasource* dv=*it;
       		++it;
       		 dv->depending_on_datasource_after_store_changed_data();
     	}



}


void hk_datasource::inform_depending_ds_insertmode(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_depending_ds_insertmode");
#endif
        list<hk_datasource*>::iterator it;
        it=p_dependinglist.begin();

	while ( it!=p_dependinglist.end() )
     	{
       		hk_datasource* dv=*it;
       		++it;
       		if (dv->is_enabled()) dv->depending_on_datasource_insertmode();
     	}



}

void hk_datasource::depending_on_datasource_insertmode(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::depending_on_datasource_insertmode");
#endif
  disable();

}



void hk_datasource::parse_sql(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::parse_sql");
#endif


 	hk_string tst_sql,original_sql;
 	tst_sql=p_original_sql;
 	original_sql=p_original_sql;
 	for (unsigned int tt=0;tt<tst_sql.size();tt++)
 	{
 	 tst_sql[tt]=toupper(tst_sql[tt]);
 	}
    	if (p_parsed_sql!=NULL)
	{
	  delete p_parsed_sql;
	  p_parsed_sql=NULL;
	}
 	 p_parsed_sql=new struct_parsed_sql;



 int 	position[6];
 int from_pos, to_pos,pos;
 int size=tst_sql.length();
 list<int> sizelist;
   pos=0;

   position[pos]=tst_sql.find("SELECT");
   if (position[pos]>-1) sizelist.insert(sizelist.end(),position[pos]);
   position[++pos]=tst_sql.find("FROM");
   if (position[pos]>-1) sizelist.insert(sizelist.end(),position[pos]);
   position[++pos]=tst_sql.find("WHERE");
   if (position[pos]>-1) sizelist.insert(sizelist.end(),position[pos]);
   position[++pos]=tst_sql.find("GROUP");
   if (position[pos]>-1) sizelist.insert(sizelist.end(),position[pos]);
   position[++pos]=tst_sql.find("HAVING");
   if (position[pos]>-1) sizelist.insert(sizelist.end(),position[pos]);
   position[++pos]=tst_sql.find("ORDER");
   if (position[pos]>-1) sizelist.insert(sizelist.end(),position[pos]);
   sizelist.insert(sizelist.end(),size);
   sizelist.sort();
   pos=0;
   if (position[pos]!=-1)
   { // SELECT PART
   	from_pos=position[pos]+6;
   	int z=pos+1;
   	to_pos=-1;

   	   	list<int>::iterator iter;
   		iter = find(sizelist.begin(),sizelist.end(),position[pos]);
   		if(iter!=sizelist.end()) iter++;
   		if (iter!=sizelist.end()) to_pos=*iter-1;

   	p_parsed_sql->select_part=original_sql.substr(from_pos,to_pos-from_pos+1);
        //FROM PART
	if (position[++pos]>-1)
   	{
   		from_pos=position[pos]+4;
   		z=pos+1;
   		to_pos=-1;
   		iter = find(sizelist.begin(),sizelist.end(),position[pos]);
   		if(iter!=sizelist.end()) iter++;
   		if (iter!=sizelist.end()) to_pos=*iter-1;
   		p_parsed_sql->from_part=original_sql.substr(from_pos,to_pos-from_pos+1);

        }
        // WHERE PART
	if (position[++pos]>-1)
   	{
   		from_pos=position[pos]+5;
   	 	z=pos+1;
   		to_pos=-1;
   		iter = find(sizelist.begin(),sizelist.end(),position[pos]);
   		if(iter!=sizelist.end()) iter++;
   		if (iter!=sizelist.end()) to_pos=*iter-1;
	   	p_parsed_sql->where_part=original_sql.substr(from_pos,to_pos-from_pos+1);
       }
	//GROUP BY PART
	if (position[++pos]>-1)
   	{
   		from_pos=position[pos]+5;
   	 	z=pos+1;
   		to_pos=-1;
   		iter = find(sizelist.begin(),sizelist.end(),position[pos]);
   		if(iter!=sizelist.end()) iter++;
   		if (iter!=sizelist.end()) to_pos=*iter-1;
	   	p_parsed_sql->groupby_part=original_sql.substr(from_pos,to_pos-from_pos+1);
   		hk_string buffer=tst_sql.substr(from_pos,to_pos);
   		z=buffer.find("BY");
   		if (z>-1) p_parsed_sql->groupby_part=original_sql.substr(from_pos,to_pos-from_pos+1);
       }

       // HAVING PART
	if (position[++pos]>-1)
   	{
   		from_pos=position[pos]+6;
   	 	z=pos+1;
   		to_pos=-1;
   		iter = find(sizelist.begin(),sizelist.end(),position[pos]);
   		if(iter!=sizelist.end()) iter++;
   		if (iter!=sizelist.end()) to_pos=*iter-1;
	   	p_parsed_sql->having_part=original_sql.substr(from_pos,to_pos-from_pos+1);
       }
       // ORDER BY PART
	if (position[++pos]>-1)
   	{
   		from_pos=position[pos]+5;
   		to_pos=-1;
   		iter = find(sizelist.begin(),sizelist.end(),position[pos]);
   		if(iter!=sizelist.end()) iter++;
   		if (iter!=sizelist.end()) to_pos=*iter-1;
   		p_parsed_sql->orderby_part=original_sql.substr(from_pos,to_pos-from_pos+1);
   		hk_string buffer=tst_sql.substr(from_pos,to_pos);
   		z=buffer.find("BY");
   		if (z>-1)
   		{
   		  p_parsed_sql->orderby_part=p_parsed_sql->orderby_part.substr(z+2,p_parsed_sql->orderby_part.size());
                 }
       }


  }


}


void 	hk_datasource::depending_on_datasource_data_has_changed()
{
   if (p_depending_on_datasource!=NULL)
   if (   p_depending_on_datasource_react_on_changed_data
       && p_depending_on_datasource->is_enabled()
       ) reload_data(true);
}



void hk_datasource::depending_on_datasource_after_store_changed_data(void)
{
   depending_on_datasource_row_change();
}


void hk_datasource::depending_on_datasource_row_change(bool take_changed_data)
{
 #ifdef HK_DEBUG
hkdebug("hk_datasource::depending_on_datasource_row_change");
#endif

 //wenn ds sich ndert muss sql statement WHERE angepasst werden
 if (p_depending_on_datasource!=NULL)
 {
   if (p_has_changed) store_changed_data();
   if ((p_depending_on_datasource->is_enabled())&&p_depending_on_datasource->mode()==mode_normal)
   	{
	   reload_data(take_changed_data);
 	}

   else
   {
     disable();
   }
 }
}

void hk_datasource::reload_data(bool take_changed_data)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::reload_data");
#endif
   create_new_sql_statement(take_changed_data);
   inform_before_disable();
   driver_specific_disable();
   p_mode=mode_disabled;
   p_enabled=false;
   if (driver_specific_enable())
     {
        p_enable_phase=true;
	setmode_normal();
        inform_visible_objects_new_columns_created();
	p_enable_phase=false;
	if (max_rows()>0)
	   {
	   	p_enabled=true;
	   	goto_first();
	   }
	   else
	   {
	   	if (!is_readonly())
	   	{
	      		p_enabled=true;
	      		setmode_insertrow();
	   	}
	   	else p_enabled=false;
	   }
     }
   if (p_enabled)inform_visible_objects_ds_enable();


}

void	hk_datasource::add_depending_fields(const hk_string& this_ds_field, const hk_string& master_ds_field,bool registerchange)
{
 #ifdef HK_DEBUG
hkdebug("hk_datasource::add_depending_fields");
#endif


  if ((this_ds_field=="")||(master_ds_field=="")) return;
   if (p_presentation!=NULL&&registerchange) p_presentation->set_has_changed();
  depending_this_fields.insert(depending_this_fields.end(),this_ds_field);
  depending_master_fields.insert(depending_master_fields.end(),master_ds_field);
 #ifdef HK_DEBUG
hkdebug("hk_datasource::add_depending_fields ENDE");
#endif

}




void hk_datasource::create_new_sql_statement(bool take_changed_data)
{
 #ifdef HK_DEBUG
hkdebug("hk_datasource::create_new_sql_statement");
#endif
 if (p_rawsql) return;
 hk_string newwhere,delimiter;
 hk_string buffer,buffer2;
  //construct the whole SQL statement;
  buffer2="SELECT "+p_parsed_sql->select_part;
  if (p_parsed_sql->from_part!="")
  	{
  	 buffer2=buffer2+" FROM ";
  	 buffer2=buffer2+p_parsed_sql->from_part;
  	 }
   buffer2+=whole_datasource_where_statement(take_changed_data);

	  if ((newwhere.length()>0)||(p_parsed_sql->where_part.length()>0))buffer2=buffer2+") ";
  if (p_parsed_sql->groupby_part!=""){buffer2=buffer2+" GROUP BY ";buffer2=buffer2+p_parsed_sql->groupby_part;}
  if (p_parsed_sql->having_part!="") {buffer2=buffer2+" HAVING ";buffer2=buffer2+p_parsed_sql->having_part;}
  if (p_sorting!="") {buffer2=buffer2+" ORDER BY ";buffer2=buffer2+p_sorting;}
  if (p_parsed_sql->orderby_part!="") {buffer2=buffer2+(p_sorting.length()==0?" ORDER BY ":" , ");buffer2=buffer2+p_parsed_sql->orderby_part;}
#ifdef HK_DEBUG
  hkdebug("BUFFER2: ",buffer2);
#endif
  buffer2+=p_sql_delimiter;

  p_sql=replace_all("%TRUE%",buffer2,p_true);
  p_sql=replace_all("%FALSE%",p_sql,p_false);
}



void hk_datasource::set_filter(const hk_string& f,bool registerchange)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::set_filter");
#endif
   if (p_presentation!=NULL&& f.size()>0&&registerchange) p_presentation->set_has_changed();

 p_filter=f;
 create_new_sql_statement();
}

void hk_datasource::set_sorting(const hk_string& s,bool registerchange)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::set_sorting");
#endif
   if (p_presentation!=NULL&& s.size()>0&&registerchange) p_presentation->set_has_changed();
 p_sorting=s;
 create_new_sql_statement();
}

hk_string hk_datasource::filter(void)
{

 return p_filter;
}

hk_string hk_datasource::sorting(void)
{
 return p_sorting;
}



void hk_datasource::clear_filter(bool registerchange)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::clear_filter");
#endif
   if (p_presentation!=NULL&& (p_filter.size()>0)&&registerchange) p_presentation->set_has_changed();
  p_filter="";
  create_new_sql_statement();
}


void    hk_datasource::clear_sorting(bool registerchange=true)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::clear_sorting");
#endif
   if (p_presentation!=NULL&& (p_sorting.size()>0)&&registerchange) p_presentation->set_has_changed();
  p_sorting="";
  create_new_sql_statement();

}


void hk_datasource::set_automatic_data_update(bool u)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::set_automatic_data_update");
#endif
 	p_automatic_data_update=u;
}



bool hk_datasource::is_automatic_data_update(void)
{
 	return p_automatic_data_update;
}

void hk_datasource::create_actual_row_where_statement(void)
{
   p_actual_row_where=create_row_where_statement_at(p_counter);
}

hk_string hk_datasource::create_row_where_statement_at(long unsigned int position,bool withwhere)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::create_row_where_statement_at");
#endif
	hk_string buffer;
	hk_string p_row_where="";
	list<hk_column*>::iterator col;
	if (p_primary_key_used)
	{  //table uses primary key
	 for(col=p_columns->begin();col!=p_columns->end();col++)
	 {
	 hk_column* cl=*col;

	 if (cl->is_primary())
	 	{
	 	  if (p_row_where==""){ if (withwhere)p_row_where=" WHERE ";}
	 	  else p_row_where+=" AND ";
                  buffer=cl->name();buffer+=" = \"";
                  buffer+=cl->transformed_asstring_at(position);
                  buffer+="\" ";
                  p_row_where+=buffer;
	 	}

	 }
	}
	else
	{ //no primary key used
	for(col=p_columns->begin();col!=p_columns->end();col++)
	 {
	  if ((*col)->fieldtype()!=hk_column::binaryfield)
	 	 {
	 	  if (p_row_where=="") {if (withwhere)p_row_where=" WHERE ";}
	 	  else p_row_where+=" AND ";

	 	  buffer=(*col)->name();
	 	  buffer+=" = \"";
	 	  buffer+=(*col)->transformed_asstring_at(position);
	 	  buffer+="\" ";
                  p_row_where+=buffer;
	 	 }
	 }
	}

	 	  #ifdef HK_DEBUG
	 	  hkdebug("ROW WHERE ",p_row_where);
	 	  #endif
 return p_row_where;
}


void hk_datasource::reset_changed_data(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::reset_changed_data");
#endif
	if (p_columns!=NULL)
	{
	 list<hk_column*>::iterator it;
	 it=p_columns->begin();
	 while (it!=p_columns->end())
	 {
	  (*it)->reset_changed_data();
	  it++;
	 }
	 set_has_not_changed();
	}
}


void hk_datasource::inform_before_row_change()
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_before_row_change");
#endif
//inform visible objects
list<hk_dsvisible*>::iterator it ;
for(it=p_visibles.begin();it!=p_visibles.end();)
  {
	hk_dsvisible* vi=*it;
       	it++;
       	vi->before_row_change();
  }

inform_depending_ds_store_data();
}

void hk_datasource::inform_visible_objects_before_store_changed_data()
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_visible_objects_before_store_changed_data");
#endif
//inform visible objects
list<hk_dsvisible*>::iterator it ;
for(it=p_visibles.begin();it!=p_visibles.end();)
  {
	hk_dsvisible* vi=*it;
       	it++;
       	vi->before_store_changed_data();
  }

}

void hk_datasource::inform_visible_objects_after_store_changed_data()
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_visible_objects_after_store_changed_data");
#endif
//inform visible objects
list<hk_dsvisible*>::iterator it ;
for(it=p_visibles.begin();it!=p_visibles.end();)
  {
	hk_dsvisible* vi=*it;
       	it++;
       	vi->after_store_changed_data();
  }

}


void	hk_datasource::inform_visible_objects_new_columns_created(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_visible_objects_new_columns_created");
#endif
//inform visible objects
list<hk_dsvisible*>::iterator it ;
for(it=p_visibles.begin();it!=p_visibles.end();)
  {
	hk_dsvisible* vi=*it;
       	it++;
       	vi->columns_new_created();
  }

}

void hk_datasource::inform_before_disable(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_before_disable");
#endif
//inform visible objects
list<hk_dsvisible*>::iterator it ;
for(it=p_visibles.begin();it!=p_visibles.end();)
  {
	hk_dsvisible* vi=*it;
       	it++;
       	vi->before_row_change();
  }


}

void hk_datasource::inform_depending_ds_disable(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_depending_ds_disable");
#endif
        list<hk_datasource*>::iterator it;
        it=p_dependinglist.begin();

	while ( it!=p_dependinglist.end() )
     	{
       		hk_datasource* dv=*it;
       		++it;

       		if (dv!=NULL)
       		{
       		  dv->disable();
       		}
     	}


}

void hk_datasource::inform_depending_ds_enable(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_depending_ds_enable");
#endif
        list<hk_datasource*>::iterator it;
        it=p_dependinglist.begin();

	while ( it!=p_dependinglist.end() )
     	{
       		hk_datasource* dv=*it;
       		++it;

       		if (dv!=NULL)
       		{
       		  dv->enable();
       		}
     	}


}
void hk_datasource::inform_visible_objects_before_columns_deleted(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_visible_objects_before_columns_deleted");
#endif
//inform visible objects
list<hk_dsvisible*>::iterator it ;
for(it=p_visibles.begin();it!=p_visibles.end();)
  {
	hk_dsvisible* vi=*it;
       	it++;
       	vi->before_columns_deleted();
  }


}


void hk_datasource::inform_visible_objects_row_add(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_visible_objects_row_add");
#endif
//inform visible objects
list<hk_dsvisible*>::iterator it ;
for(it=p_visibles.begin();it!=p_visibles.end();)
  {
	hk_dsvisible* vi=*it;
       	it++;
       	vi->row_added();
  }


}
void hk_datasource::inform_visible_objects_row_delete(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::inform_visible_objects_row_delete");
#endif
//inform visible objects
list<hk_dsvisible*>::iterator it ;
for(it=p_visibles.begin();it!=p_visibles.end();)
  {
	hk_dsvisible* vi=*it;
       	it++;
       	vi->row_deleted();
  }


}

void hk_datasource::set_sqldelimiter(const hk_string& d)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::set_sqldelimiter");
#endif
  p_sql_delimiter=d;

}

hk_database* hk_datasource::database(void)
{
   return p_database;
}

#ifdef HK_DEBUG
void hk_datasource::dump_data()
{
list<hk_column*>::iterator it;
cout<<endl<<"DUMP"<<endl;
cout<<"===="<<endl;

for (unsigned int k=0;k<max_rows();k++)
{
  int hcol=0;
  for (it=columns()->begin(); it!=columns()->end();it++)
  {
   cout <<k<<" "<<hcol++<<" "<<(*it)->asstring_at(k)<<" ";
  }
  cout<<endl;
}
cout <<"================="<<endl<<endl;
}

#endif


void  hk_datasource::savedata(ofstream& s,bool saveall)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::savedata");
#endif
       hk_string dtag="DATASOURCE";
	start_mastertag(s,dtag);

	hk_string dd=(type()==ds_table?"TABLE":"QUERY");
        set_tagvalue(s,"TYPE",dd);
        set_tagvalue(s,"DATASOURCENAME",name());
        set_tagvalue(s,"SQL",sql());
    if (saveall)
    {
      hk_string mtag="DATASOURCEEXTRA";
	start_mastertag(s,mtag);
	set_tagvalue(s,"FILTER",p_filter);
        set_tagvalue(s,"SORTING",p_sorting);
        set_tagvalue(s,"DATETIMEFORMAT",p_datetimeformat);
        set_tagvalue(s,"DATEFORMAT",p_dateformat);
        set_tagvalue(s,"TIMEFORMAT",p_timeformat);
       if (p_presentation!=NULL)
       {
       	set_tagvalue(s,"UPN",p_dscounter);
       	set_tagvalue(s,"DEPENDING_ON",p_depending_on_presentationdatasource);
	set_tagvalue(s,"REACT_ON_CHANGED_DATA",p_depending_on_datasource_react_on_changed_data);
	hk_string dm;
	switch ( p_dependingmode)
	{
	  case depending_nohandle 	: dm="NOHANDLE";
	  				break;
	  case depending_change 	: dm="CHANGE";
	  				break;
	  case depending_delete 	: dm="DELETE";
	  				break;
	  case depending_changedelete 	: dm="CHANGEDELETE";
	  				break;
	  default 		  	: dm="STANDARD";


	}
	set_tagvalue(s,"DEPENDINGMODE",dm);
       }
       hk_string mtag1="DEPENDINGFIELDS";
	start_mastertag(s,mtag1);
        list<hk_string>::iterator thisit=depending_this_fields.begin();
        list<hk_string>::iterator masterit=depending_master_fields.begin();
        while (thisit!=depending_this_fields.end())
        {
        set_tagvalue(s,"THISFIELD",(*thisit));
        set_tagvalue(s,"MASTERFIELD",(*masterit));

         thisit++;
         masterit++;
        }
       end_mastertag(s,mtag1);
       end_mastertag(s,mtag);
    }

       end_mastertag(s,dtag);

}


/*bool hk_datasource::savequerytofile(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::savequerytofile");
#endif
  if (name().size()==0) if (!ask_name())return false;
 ofstream* p_save=p_database->savestream(name(),ft_query);
 if (p_save==NULL) return false;
 savedata(*p_save,false);
 delete p_save;
 return true;
}*/

/*bool hk_datasource::loadqueryfromfile(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::loadqueryfromfile");
#endif
 hk_string res=p_database->load(name(),ft_query);
 if (res.size()==0)return false;
 loaddata(res);
 return true;
} */

void  hk_datasource::loaddata(const hk_string& definition)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::loaddata");
#endif
        hk_string buffer;
        hk_string buffer2;
        hk_string value;
        get_tagvalue(definition,"TYPE",buffer);
        get_tagvalue(definition,"DATASOURCENAME",value);
        set_name(value);
        if (!((type()==ds_table)&&(buffer=="TABLE")))
        {
           get_tagvalue(definition,"SQL",p_original_sql);
           p_sql=p_original_sql;
 	   parse_sql();
        }

        get_tagvalue(definition,"FILTER",p_filter);
        get_tagvalue(definition,"SORTING",p_sorting);
        get_tagvalue(definition,"DATETIMEFORMAT",p_datetimeformat);
        get_tagvalue(definition,"DATEFORMAT",p_dateformat);
        get_tagvalue(definition,"TIMEFORMAT",p_timeformat);
        get_tagvalue(definition,"DEPENDINGFIELDS",buffer);
       if (p_presentation!=NULL)
       {
        get_tagvalue(definition,"UPN",p_dscounter);
        get_tagvalue(definition,"DEPENDING_ON",p_depending_on_presentationdatasource);
	get_tagvalue(definition,"REACT_ON_CHANGED_DATA",p_depending_on_datasource_react_on_changed_data);
        int i=1;
        clear_depending_fields();
        while (get_tagvalue(buffer,"THISFIELD",value,i))
        {
          get_tagvalue(buffer,"MASTERFIELD",buffer2,i);
          add_depending_fields(value,buffer2);
          i++;
        }
        if (!get_tagvalue(buffer,"THISFIELD",value,1))
        	set_depending_on(NULL);

	hk_string dm;
	get_tagvalue(definition,"DEPENDINGMODE",dm);
	if (dm=="NOHANDLE") p_dependingmode=depending_nohandle;
	else
	if (dm=="CHANGE") p_dependingmode=depending_change;
	else
	if (dm=="DELETE") p_dependingmode=depending_delete;
	else
	if (dm=="CHANGEDELETE") p_dependingmode=depending_changedelete;
	else
	 p_dependingmode=depending_standard;

       }
	create_new_sql_statement();

}




bool hk_datasource::ask_name(void)
{
 #ifdef HK_DEBUG
hkdebug("hk_datasource::ask_name");
#endif
 hk_string infotext;
 if (type()==ds_query) infotext=hk_translate("Please enter the queryname:");
 else infotext=hk_translate("Please enter the tablename:");
 hk_string res=show_stringvaluedialog(infotext);
  if (res.size()==0) return false;

   set_name(res);

  return true;

}

bool hk_datasource::alter_column(const hk_string& col, const hk_string* name,hk_column::enum_fieldtype* newtype,long* size,const hk_string* defaultvalue,const bool* primary,const bool* notnull)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::alter_column");
#endif
  if (p_mode!=mode_altertable||col.size()==0)return false;
   hk_column* acol=column_by_name(col);
   if (acol==NULL)return false;
   class_altercolumns s;
   s.name=col;
   if (name!=NULL)s.newname=*name;else s.newname=col;
   if(newtype!=NULL)s.type=*newtype; else s.type=hk_column::otherfield;
   if(size!=NULL)s.size=*size; else s.size=-1;
   if (defaultvalue!=NULL) s.defaultvalue=*defaultvalue; else s.defaultvalue="HK_NULL";
   if (primary!=NULL) s.primary=*primary; else s.primary=acol->is_primary();
   if (notnull!=NULL) s.notnull=*notnull; else s.notnull=acol->is_notnull();
   p_altercolumns.insert(p_altercolumns.end(),s);
   return false;
}

bool hk_datasource::delete_column(const hk_string& col)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::delete_column");
#endif

 if (p_mode!=mode_altertable||col.size()==0)return false;
 p_deletecolumns.insert(p_deletecolumns.end(),col);
 return true;
}

bool hk_datasource::alter_table_now(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::alter_table_now");
#endif
 if (p_mode==mode_altertable)
 {
    bool res=driver_specific_alter_table_now();
    if (res)
    {
    clear_columnlist();
    inform_when_table_structure_changes();
    }
    p_mode=mode_disabled;
    return res;
 }
 else return false;
}

void hk_datasource::setmode_altertable(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::setmode_altertable");
#endif
 if (!p_database->table_exists(name())||type()!=ds_table) return;
 if (is_enabled()) disable();
    clear_columnlist();
    clear_modecolumnlists();
    p_mode=mode_altertable;
}

list<hk_datasource::indexclass>* hk_datasource::indices(void)
{

   return driver_specific_indices();
}


bool	hk_datasource::drop_index(const hk_string& i)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::drop_index");
#endif
    bool res= driver_specific_drop_index(i);
    if (res)inform_when_indexlist_changes();
    return res;
}
bool    hk_datasource::create_index(const hk_string& name,bool unique,list<hk_string>& fields)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::create_index");
#endif
   bool res= driver_specific_create_index(name,unique,fields);
#ifdef HK_DEBUG
 if (res) hkdebug("index created");
 else hkdebug("index NOT created");
#endif
    if (res)inform_when_indexlist_changes();
    return res;
}

bool    hk_datasource::alter_index(const hk_string& name,bool unique,list<hk_string>& fields)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::alter_index");
#endif

   drop_index(name);
   return create_index(name,unique,fields);

}

bool	hk_datasource::driver_specific_goto_row(unsigned long r)
{
return false;
}
bool	hk_datasource::driver_specific_drop_index(const hk_string& i)
{
return false;
}

bool    hk_datasource::driver_specific_create_index(const hk_string& name,bool unique,list<hk_string>& fields)
{
return false;
}

list<hk_string>* hk_datasource::depending_on_thisfields(void)
{
   return &depending_this_fields;
}

list<hk_string>* hk_datasource::depending_on_masterfields(void)
{
   return &depending_master_fields;
}

bool hk_datasource::datasource_used(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::datasource_used");
#endif
  bool res =false;
  if (p_visibles.size()>0||p_dependinglist.size()>0) res=true;
  return res;

}

void hk_datasource::set_has_changed(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::set_has_changed");
#endif
 p_has_changed=true;
 inform_depending_ds_data_has_changed();
}

void hk_datasource::set_has_not_changed(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::set_has_not_changed");
#endif
  p_has_changed=false;
}

bool hk_datasource::depending_on_react_on_data_changes(void)
{
   return p_depending_on_datasource_react_on_changed_data;
}

hk_datasource::enum_dependingmodes hk_datasource::dependingmode(void)
{
    return p_dependingmode;
}


bool hk_datasource::depending_on_datasource_updaterow_ok(void)
{
    if(p_depending_on_datasource==NULL) return true;
    switch (p_dependingmode)
    {
       case depending_change:
       case depending_changedelete:
       case depending_nohandle:  return true;
       				break;
       case depending_standard: if  (max_rows()>0)
       					{
					  list<hk_string>::iterator it=depending_master_fields.begin();
					  while (it!=depending_master_fields.end())
					  {
					     hk_column* c =p_depending_on_datasource->column_by_name((*it));
					     if (c!=NULL) if (c->has_changed()) return false;
					     it++;
					  }
					}else return true;
       				break;
       default: return false;
    }
  return true;
}

void hk_datasource::depending_on_datasource_before_update_row(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::depending_on_datasource_before_update_row");
#endif
if(p_depending_on_datasource==NULL) return;
if (p_dependingmode!=depending_change
  &&p_dependingmode!=depending_changedelete)
      return;
list<hk_string>::iterator it= depending_master_fields.begin();
list<hk_string>::iterator thisit= depending_this_fields.begin();
hk_string changelist;
while (it!=depending_master_fields.end())
{
   hk_column* col =p_depending_on_datasource->column_by_name((*it));
   if (col!=NULL)
      {
	if (col->has_changed())
	{
	   if (changelist.size()==0) changelist=" SET ";
	   else changelist+=", ";
	   changelist += (*thisit);
	   changelist+=" = \"";
	   changelist+= col->changed_data_asstring();
	   changelist+="\" ";

	}

      }
   it++;thisit++;
}

if (changelist.size()==0) return;
hk_string s= "UPDATE ";
s+=name()+changelist+whole_datasource_where_statement();
    hk_actionquery* a = p_database->new_actionquery();
    if (a==NULL) return;
    a->set_sql(s.c_str(),s.size());
    a->execute();
    delete a;
// cout <<"beforeupdaterow SQL: "<<s<<endl;
}



void hk_datasource::depending_on_datasource_after_update_row(void)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::depending_on_datasource_after_update_row");
#endif
   reload_data();
}


bool hk_datasource::delete_rows(unsigned long from,unsigned long to)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::delete_rows(unsigned long from,unsigned long to)");
#endif
cout <<"delete_rows" <<endl;
    if (max_rows()==0||type()!=ds_table)
       {
//	  cout <<"pcounter: "<<p_counter<<endl<<"type: "<<type()<<endl;
	  return false;
       }
    unsigned long f,t,f_orig;
    if (from>to)
    	     {
		t=from;
		f=to;
    	     }
	else
	     {
		t=to;
		f=from;
	     }
    if (f>=max_rows()) return false;
    if (t>=max_rows()) t=max_rows()-1;
    hk_string w,b;
    f_orig=f;
    while (f<=t)
    {
     b=create_row_where_statement_at(f,false);
     if (b.size()>0)
     {
      if (w.size()!=0) w+=" ) OR ( ";
      w+=b;
     }
     f++;
    }
    if (w.size()==0) return true;

    hk_actionquery* a = p_database->new_actionquery();
    if (a==NULL) return false;

    hk_string query="DELETE FROM ";
    query+=name()+" WHERE (";
    query+=w;
    query+=") ";
    query+=p_sql_delimiter;
 cout <<"rangequery: "<<query<<endl;
    a->set_sql(query.c_str(),query.size());
    bool res=a->execute();
    delete a;
    unsigned long pp=p_counter;
    if (res)
      {  cout <<"f= "<<f_orig<<" t: "<<t<<endl;
         while (t>=f_orig)
	 {
	  driver_specific_delete_data_at(t); cout <<"deleteat: "<<t<<endl;
	  t--;
	 }

      } else cout <<"res==false!!!!!!!!!!!"<<endl;
    inform_visible_objects_row_delete();
    goto_row(pp);
    return res;
}


hk_string hk_datasource::whole_datasource_where_statement(bool take_changed_data)
{
#ifdef HK_DEBUG
hkdebug("hk_datasource::whole_datasource_where_statement");
#endif
 hk_string newwhere,delimiter;
 hk_string buffer,buffer2;
   if (p_parsed_sql->where_part!="")
          {
           newwhere=" WHERE (";
           newwhere=newwhere+p_parsed_sql->where_part;
          }

if (p_depending_on_datasource!=NULL)
 {
  list<hk_string>::iterator this_ds;
  list<hk_string>::iterator master_ds;
  master_ds=depending_master_fields.begin();

  for (this_ds=depending_this_fields.begin();this_ds!=depending_this_fields.end();this_ds++)
  {
    if (newwhere=="")
    {
       	if(p_parsed_sql->where_part=="")newwhere=" WHERE (" ;
   	else newwhere=") AND (";
    }
    else newwhere=newwhere+") AND (";
    hk_column* buffercolumn=p_depending_on_datasource->column_by_name((*master_ds));
    if (buffercolumn!=NULL)
   	{
	if (take_changed_data&&buffercolumn->has_changed()&&buffercolumn->transformed_changed_data()!=NULL)
	     buffer2=buffercolumn->transformed_changed_data()->data;
	else
	     buffer2=buffercolumn->transformed_asstring();
   	delimiter=buffercolumn->sql_column_delimiter();
   	}
   	else
   	{
   	  buffer2="";
   	  delimiter="\"";
   	}
    buffer=*this_ds+" =";
    buffer=buffer+delimiter;
    buffer=buffer+buffer2+delimiter;
    master_ds++;
    newwhere=newwhere+buffer;

 }


 }//endif depending_on_datasource !=NULL

//add filter if defined
 if (p_filter.length()>0)
 {
       	if(newwhere=="")newwhere=" WHERE (";
       	else newwhere=newwhere+") AND (";
        newwhere=newwhere+p_filter;
 }

 if ((newwhere.length()>0))newwhere+=" ) ";

 return newwhere;
}


void hk_datasource::transaction_begin(hk_string name)
{
  driver_specific_transaction_begin(name);
}

void hk_datasource::transaction_commit(hk_string name)
{
  driver_specific_transaction_commit(name);
}

void hk_datasource::transaction_rollback(hk_string name)
{
  driver_specific_transaction_rollback(name);
}

void hk_datasource::driver_specific_transaction_begin(hk_string)
{
}

void hk_datasource::driver_specific_transaction_commit(hk_string)
{
}

void hk_datasource::driver_specific_transaction_rollback(hk_string)
{
}
