// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.3 $
#include "hk_drivermanager.h"
#include <dlfcn.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <iostream.h>
#include "hk_connection.h"


requestdialogtype* hk_drivermanager::p_driverselectdialog=NULL;
hk_string hk_drivermanager::p_hk_classespath="";
vector<hk_string> hk_drivermanager::p_driverlist;

typedef hk_connection*  connectiontype(hk_drivermanager* );

hk_drivermanager::hk_drivermanager(bool runtime):hk_class()
{
  char* res=getenv("HK_CLASSES");
  if (res==NULL) p_hk_classespath="/usr/local/hk_classes";
  else p_hk_classespath=res;
  p_hk_classespath=p_hk_classespath+"/drivers";
  scan_directory();
  hk_string classespath=getenv("HOME");
  classespath +="/.hk_classes";
  mkdir (classespath.c_str(),S_IRUSR|S_IWUSR|S_IXUSR);
  load_preferences();
  p_runtime_only=runtime;
}

hk_drivermanager::~hk_drivermanager(void)
{
    list<hk_connection*>::iterator it= p_connections.begin();
    while (it!=p_connections.end())
    {
     hk_connection* con=(*it);
     it++;
     delete con;
    }
}

hk_connection* hk_drivermanager::new_connection(const hk_string& drivername)
{
#ifdef HK_DEBUG
hkdebug("hk_drivermanager::new_connection");
#endif
 hk_connection* 	n=NULL;
 void 			*handle;
 connectiontype *connection;
 hk_string d;
 hk_string driver;
 char* error;
  if (drivername.size()==0) driver=driverselect();
  else driver=drivername;




 d=p_hk_classespath+"/libhk_";
 d=d+driver;
 d=d+"driver.so";
#ifdef HK_DEBUG
hkdebug("Driver Path: ",d);
#endif

     if ((handle=dlopen(d.c_str(),RTLD_LAZY)))
     {

#ifdef HK_DEBUG
hkdebug("hk_drivermanager::new_connection handle=true");
#endif
      connection=(connectiontype*)dlsym(handle,"create_connection");

      if ((error=dlerror())!=NULL||connection==NULL)
        {
         show_warningmessage(hk_translate("Internal error: Database driver not compatibel"));
        }
      else
      {
#ifdef HK_DEBUG
hkdebug("hk_drivermanager::new_connection (*connection)(this)");
#endif
        n= (*connection)(this);
      }
     }
     else
       {
        show_warningmessage(hk_translate("Database driver could not be loaded"));
       return NULL;
       }

     hk_string classespath=getenv("HOME");
     classespath +="/.hk_classes/";
     classespath +=driver;
     mkdir (classespath.c_str(),S_IRUSR|S_IWUSR|S_IXUSR);
     if (n!=NULL)
       {
          n->set_classespath(classespath);
	  p_connections.insert(p_connections.end(),n);
       }
#ifdef HK_DEBUG
hkdebug("hk_drivermanager::new_connection ENDE");
#endif

   return n;


}

void hk_drivermanager::set_path(const hk_string& path)
{
     p_hk_classespath=path;
     scan_directory();

}


vector<hk_string>* hk_drivermanager::driverlist(void)
{
 return &p_driverlist;


}

hk_string hk_drivermanager::path(void)
{
   return p_hk_classespath;
}



void hk_drivermanager::scan_directory(void)
{

     DIR *dp;
     struct dirent *entry;

     hk_string datei;
       p_driverlist.erase(p_driverlist.begin(),p_driverlist.end());

     if ((dp=opendir(p_hk_classespath.c_str()))!=NULL)
     {

     while ((entry = readdir(dp))!=NULL)
     {
       datei=entry->d_name;
       size_t p =datei.find("driver.so");
       if (p<datei.size()&&(datei.size()-9)==p)
       {
          datei.replace(p,datei.size()-p,"");
           p =datei.find("libhk_");
           if (p==0)
            {
             datei.replace(0,6,"");
             p_driverlist.insert(p_driverlist.end(),datei);
            }
       }
     }
   }
}


void hk_drivermanager::set_driverselectdialog(requestdialogtype* dialog)
{

   hk_drivermanager::p_driverselectdialog=dialog;
}


hk_string hk_drivermanager::driverselect(void)
{
   if (p_driverselectdialog!=NULL)
   {
       return p_driverselectdialog();
    //   return 0;
   }
   else
    {
      hk_string result;
      cout<<endl<< 		  "==========================================="<<endl;
      cout <<endl<<hk_translate("The following database drivers were found:")<<endl;
      cout<< 		  "==========================================="<<endl;
      vector<hk_string>::iterator it=p_driverlist.begin();
      int i=1;
      while (it!=p_driverlist.end())
      {
       cout <<"("<<i++<<") "<<(*it)<<endl;
       it++;
      }
      cout <<endl<<"("<<i<<") "<<hk_translate("Select directory")<<endl;
      cout<< 		  "==========================================="<<endl;
      cout <<endl<<hk_translate("Please select: ");
      char c;
      hk_string r;
      int k;
      do
      {
         cin.get(c);
         r=c;
         k=atoi(r.c_str());
      }
      while(k<1 || k>i);
      int y=1;
      it=p_driverlist.begin();
      if (k==i)
      {
        hk_string v;
        cout<<endl<< 		  "==========================================="<<endl;
         cout <<hk_translate("Enter new Driver Path")<<endl;
         cout <<hk_translate("Actual Path is: ")<<p_hk_classespath<<endl;
         cout<< 	"==========================================="<<endl;
         cout <<hk_translate("New Path: ");
        cin>>v;
        set_path(v.c_str());
         result=driverselect();
      }
      else
      {
       result=p_driverlist[k-1];
      }
      return  result.c_str();
    }

}

void	hk_drivermanager::connection_remove(hk_connection* c)
{
  if (c!=NULL) p_connections.remove(c);

}

void hk_drivermanager::load_preferences(void)
{
#ifdef HK_DEBUG
hkdebug("hk_drivermanager::load_configuration");
#endif
//cout <<"load_configuration"<<endl;
  hk_string p_classespath=getenv("HOME");
  p_classespath +="/.hk_classes";
   hk_string n=p_classespath+"/preferences";
   long size;
   FILE *in;  //cout <<"Lade Datei:"<<n<<endl;
   in=fopen(n.c_str(),"r");
   if (in)
   {
      fseek(in,0,SEEK_END);
      size=ftell(in);
      fseek(in,0,SEEK_SET);
      char* p_new_data=new char[size];
      fread(p_new_data,size,1,in);
      hk_string b;
  	if (get_tagvalue(p_new_data,"DEFAULTTIMEFORMAT",b)) p_defaulttimeformat=b;
  	if (get_tagvalue(p_new_data,"DEFAULTDATETIMEFORMAT",b)) p_defaultdatetimeformat=b;
  	if (get_tagvalue(p_new_data,"DEFAULTDATEFORMAT",b)) p_defaultdateformat=b;
      delete[] p_new_data;
   }

}
