// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.19 $

#include "hk_presentation.h"
#include "hk_database.h"
#include "hk_datasource.h"
#include "hk_definitions.h"
#include "hk_dsvisible.h"
#include "hk_form.h"



hk_presentation::hk_presentation(void):hk_dsvisible(this)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::hk_presentation");
#endif
  p_database=NULL;
  p_has_changed=false;
  set_mode(hk_presentation::view);
  p_counter=0;
  p_presentationtype=hk_presentation::general;

}

hk_presentation::~hk_presentation(void)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::~hk_presentation");
#endif
  if (p_datasource!=NULL)
    {
      p_datasource->visible_remove(this);
      p_datasource->p_presentation=NULL;
    }
  clear_datasourcelist();
  p_datasource=NULL;
  if (p_database!=NULL) p_database->presentation_remove(this);
}

void hk_presentation::set_database(hk_database* db)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::set_database");
#endif

  if (p_database!=NULL)
  	{
  	 p_database->presentation_remove(this);
  	 p_database=NULL;
  	}
  if (db!=NULL)
    	{
    		db->presentation_add(this);
    	}

  p_database=db;



}


long hk_presentation::add_datasource(hk_datasource* d)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::add_datasource");
#endif
   if (d==NULL) return -1;
   d->p_dscounter=p_counter;
   p_counter++;
   p_datasources.insert(p_datasources.end(),d);
   return d->p_dscounter;
}




long	hk_presentation::new_datasource(const hk_string& name,bool query)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::new_datasource");
#endif
    if (p_database==NULL) return -1;
    hk_datasource* d=p_database->load_datasource(name,query,this);
    set_has_changed();
    return add_datasource(d);
}

hk_datasource*	hk_presentation::get_datasource(long nr)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::get_datasource(long)");
#endif
   list<hk_datasource*>::iterator it=p_datasources.begin();
   while (it!=p_datasources.end())
   {
     if ( (*it)->p_dscounter==nr) return (*it);
     it++;
   }
   return NULL;
}

hk_datasource*	hk_presentation::get_datasource(const hk_string& description)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::get_datasource(const hk_string&)");
#endif
   list<hk_datasource*>::iterator it=p_datasources.begin();
   while (it!=p_datasources.end())
   {
     if ( unique_datasourcename((*it)->p_dscounter)==description) return (*it);
     it++;
   }
   return NULL;
}

void	hk_presentation::remove_datasource(long nr)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::remove_datasource");
#endif
   remove_datasource(get_datasource(nr));
}

void hk_presentation::remove_datasource( hk_datasource* d)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::remove_datasource");
#endif
  if (d!=NULL)
  {
    p_datasources.remove(d);
    set_has_changed();
  }
}

void hk_presentation::clear_datasourcelist(void)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::clear_datasourcelist");
#endif
   list<hk_datasource*>::iterator it=p_datasources.begin();
   while (it!=p_datasources.end())
   {
     hk_datasource* d=(*it);
     it++;
     if (d!=NULL)
     {
      d->store_changed_data();
      d->disable();
      d->p_presentation=NULL;
     }

//     if (d!=nodelete)delete d;
   delete d;
   }

   p_datasources.clear();
#ifdef HK_DEBUG
hkdebug("hk_presentation::clear_datasourcelist ENDE");
#endif
}

void 	hk_presentation::savedata(ofstream& s)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::savedata");
#endif
     hk_string tag="PRESENTATION";
     start_mastertag(s,tag);
   list<hk_datasource*>::iterator it=p_datasources.begin();
   while (it!=p_datasources.end())
   {
     (*it)->savedata(s);
     it++;
   }



end_mastertag(s,tag);
}

void 	hk_presentation::loaddata(const hk_string& definition)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::loaddata");
#endif
	hk_string buffer2;
	hk_string value;
	int i=1;
	clear_datasourcelist();
	while (get_tagvalue(definition,"DATASOURCE",value,i))
        {
	  hk_datasource* ds=NULL;
          get_tagvalue(value,"TYPE",buffer2,i);
	  hk_string dsname;
          get_tagvalue(value,"DATASOURCENAME",dsname,i);
	  bool pquery=true;
	  if (buffer2=="TABLE")  pquery=false;
	  long int dint=new_datasource(dsname,pquery);
	  ds=get_datasource(dint);
	  if (ds!=NULL)
	  	{
			ds->loaddata(value);
			if (ds->p_dscounter>=p_counter) p_counter=ds->p_dscounter+1;
	  	}
	  i++;
        }

}

hk_string hk_presentation::unique_datasourcename(long nr)
{
 hk_string result;
 hk_datasource* rd =get_datasource(nr);
 if (rd!=NULL)
 {
     result=rd->name()+" (";
     char* m= new char[50];
     sprintf(m,"%ld",rd->p_dscounter);
     result+=m;

     delete[] m;
 }
 result+=") ";
 hk_datasource* depds=get_datasource(rd->depending_on_presentationdatasource());
 if (depds!=NULL)
 {
    result+=" [";
    result+=depds->name()+"]";

 }
 if (rd->filter().size()>0)
 {
    result+=" | ";
    result+=rd->filter();
 }
 return result;
}

void hk_presentation::set_has_changed(void)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::set_has_changed");
#endif
  if (p_mode==design)
      p_has_changed=true;
}

bool hk_presentation::has_changed(void)
{
	return p_has_changed;
}


void hk_presentation::set_mode(enum_mode s)
{

	if (s==hk_presentation::design && runtime_only()) return;
	p_mode=s;

	bool enable=(s==hk_presentation::view);
	list<hk_datasource*>* mydatasources=datasources();
	list<hk_datasource*>::iterator it = mydatasources->begin();
	while (it!=mydatasources->end())
	{
	 if (enable)
	 	{
		  if ((*it)->datasource_used())(*it)->enable();
		}
	 else (*it)->disable();
	 it++;
	}




	widget_specific_modechanges(s);

}


void hk_presentation::reset_has_changed(void)
{
  p_has_changed=false;
}


void hk_presentation::database_delete(void)
{
    database_has_vanished();
    p_database =NULL;
}


void hk_presentation::database_has_vanished(void)
{

}


void hk_presentation::before_source_vanishes(void)
{
   hk_dsvisible::before_source_vanishes();
    p_datasources.remove(datasource());
}


bool hk_presentation::ask_name(void)
{
#ifdef HK_DEBUG
hkdebug("hk_presentation::ask_name");
#endif
 hk_string res=show_stringvaluedialog(hk_translate("Please enter the name of the form:"));
  if (res.size()==0) return false;

  set_name(res);

  return true;

}

