// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
// $Revision: 1.7 $
#include "hk_report.h"
#include "hk_reportsection.h"
#include "hk_reportdata.h"
#include <time.h>
#include "hk_reportutils.h"
#include "hk_column.h"
#include "hk_datasource.h"
#include "hk_database.h"




//********************************************
//**   hk_report                            **
//********************************************



hk_report::hk_report(void):hk_presentation()
{
  p_recodefunction=NULL;
  p_output=&cout;
  p_filestream=NULL;
  p_pagehead= new hk_reportsection(this);
  p_pagehead->set_automatic_create_data(false);
  p_pagefoot= new hk_reportsection(this);
  p_pagefoot->set_automatic_create_data(false);
  p_multiplefiles=false;
  p_pagenamecolumn=NULL;
  p_includeperiodic=0;
  p_rowcounter=0;
  p_masterreport=NULL;
  p_newpage_on_periodic=false;
  p_pagenumber=p_startpagenumber=1;
  p_absolutepagenumber=1;
  p_reportname="unbenannt";
//  wanna_debug(true);
}




hk_report::~hk_report(void)
{
  vector<hk_reportsection*>::iterator it=p_sections.begin();
  while(it!=p_sections.end())
    {
      delete (*it);
      it++;
    }
}

void hk_report::set_filename(const hk_string& f)
{
  p_originalfilename=p_filename=f;


}

hk_string hk_report::filename(void)
{
   return p_filename;
}


void	hk_report::set_begin(const hk_string& b)
{
    p_begin=b;
}
hk_string	hk_report::begin(void)
{
  return p_begin;
}
void		hk_report::set_end(const hk_string& e)
{
  p_end=e;

}
hk_string	hk_report::end(void)
{
    return p_end;
}

hk_reportsection* hk_report::new_section(void)
{
  hk_reportsection* s=new hk_reportsection(this);
  p_sections.insert(p_sections.end(),s);
  s->set_presentationdatasource(presentationdatasource());
  return s;

}

hk_reportsection* hk_report::section_at(unsigned int a)
{
 if (a<p_sections.size()) return p_sections[a];
 else return NULL;
}

/*bool hk_report::widget_specific_row_change()
{
            char* m= new char[50];
    	 sprintf(m,"%ld",datasource()->row_position());

         p_rownumber=m;
         delete[] m;
         return true;

} */




bool hk_report::execute(void)
{
#ifdef HK_DEBUG
hkdebug("execute");
#endif
 //prepare datasource for output, resort data
   if (datasource()==NULL) return false;

    p_currenttime=time(NULL);
    hk_string ds_sort=datasource()->sorting();
    hk_string sort;
    vector<hk_reportsection*>::iterator sit=p_sections.begin();
    while (sit!=p_sections.end())
       {
         if ((*sit)->unique())
         {
           hk_string st=(*sit)->unique_column();
           if (sort.find(st)>sort.size())
           {
             if (sort.size()>0)sort+=" , ";

             sort+=st;
           }
         }
         sit++;
       }
     if ((sort.size()>0)&&(ds_sort.size()>0)) sort+=" , ";
     sort+=ds_sort;
     datasource()->set_sorting(sort);
     datasource()->disable();
     datasource()->enable();
     if (!datasource()->is_enabled())
     {
      show_warningmessage(hk_translate("Datasource of this report could not be enabled"));
      return false;
     }
     p_pagenamecolumn=datasource()->column_by_name(p_pagename);
     if (p_pagenamecolumn!=NULL)
        {
           p_pagenamecolumn->set_datetimeformat(datetimeformat());
           p_pagenamecolumn->set_dateformat(dateformat());
           p_pagenamecolumn->set_timeformat(timeformat());
        }
     new_outputstream();
//start output
     p_rowcounter=0;
     p_pagenumber=p_startpagenumber;
     p_absolutepagenumber=1;
     if (p_masterreport==NULL)
       {
         if (p_output==&cout)*p_output<<p_filebegin;
         p_pagehead->actual_hk_string();
       }
             *p_output<<  replace(p_begin);
     unsigned int k=0;
     do
     {
       vector<hk_reportsection*>::iterator it=p_sections.begin();
      while (it!=p_sections.end())
       {
	(*it)->actual_hk_string();
	 sit=it;
	 sit++;
	 p_lastpage=(k==datasource()->max_rows()-1);
	 if ((*it)->new_page_after_section()&&(*it)->section_was_printed()) new_page();
         it++;
       }
       datasource()->goto_next();
       k++;
     } while(k<datasource()->max_rows());
   datasource()->set_sorting(ds_sort);


     (*p_output) << replace(p_end);
     if (p_masterreport==NULL)
     {
        p_pagefoot->actual_hk_string();
        *p_output<<p_pagedelimiter;
        if (p_output==&cout)*p_output<<p_fileend;
     }
     if (p_masterreport==NULL)
     {
       if (p_filestream!=NULL)
       {
	 p_filestream->close();
         delete p_filestream;
       }
       p_output=&cout;
     }
    return true;
}




void hk_report::new_outputstream(void)
{
     if (p_filestream!=NULL)
     {
        *p_filestream<<p_fileend;
       delete p_filestream;
       p_filestream=NULL;
     }

     if (p_masterreport!=NULL)
        {
         p_output=p_masterreport->outputstream();
         return;
        }
   char* p_pagenumberhk_string=new char[60];
      sprintf(p_pagenumberhk_string,"%ld",p_pagenumber);

   if (p_filename.size()>0)
      {
   	if (p_multiplefiles)
       		{
       		  if (p_pagenamecolumn==NULL)
       		      p_filename=p_originalfilename+p_pagenumberhk_string;
       		  else
       		      p_filename=p_pagenamecolumn->asstring_at(datasource()->row_position()+1);
       		  p_filename+=p_fileextension;

       		}
       		else p_filename=p_originalfilename+p_fileextension;
         delete[] p_pagenumberhk_string;
         p_filestream=new ofstream(p_filename.c_str());
         *p_filestream<<p_filebegin;
          p_output=p_filestream;
          p_absolutepagenumber=1;
          if (p_output!=NULL) return;
      }

    p_output=&cout;

}

void hk_report::new_page(void)
{
if (p_masterreport!=NULL)
    {
       p_masterreport->new_page();
       p_output=p_masterreport->outputstream();
       return;
    }

if (!p_lastpage )
{
   p_pagefoot->actual_hk_string();
   *p_output << p_pagedelimiter;
   p_pagenumber++;
   p_absolutepagenumber++;
   set_pagenumber(p_pagenumber);
   if(p_multiplefiles) new_outputstream();
   p_pagehead->actual_hk_string();
}
}

void hk_report::set_pagenumber(long f)
{
  p_pagenumber=f;
}

void hk_report::set_firstpagenumber(long f)
{
   set_pagenumber(f);
   p_startpagenumber=f;

}


void hk_report::set_pagenamecolumn(const hk_string& c)
{
   p_filename=p_pagename=c;
   if (p_pagename.size()==0) p_multiplefiles=false;
      else p_multiplefiles=true;
}



bool hk_report::datasource_disable(void)
{
 p_pagenamecolumn=NULL;
 vector<hk_reportsection*>::iterator it=p_sections.begin();
 while (it!=p_sections.end())
 {
  (*it)->reset();
  it++;

 }

 return true;

}
bool hk_report::before_columns_deleted(void)
{
 vector<hk_reportsection*>::iterator it=p_sections.begin();
 while (it!=p_sections.end())
 {
  (*it)->clear_countingfields();
  it++;

 }
 return datasource_disable();
}



void hk_report::rowcount(long c)
{
// if (p_rowcounter==0) p_rowcounter+=c;
if (p_masterreport!=NULL)
  {
   p_masterreport->rowcount(c);
   return;
  }
 p_rowcounter+=c;
   if ((p_includeperiodic>0)&&(p_rowcounter>(signed)p_includeperiodic))
             {
               *p_output<<p_includeperiodichk_string;
	        p_rowcounter=c;
               if (p_newpage_on_periodic)
               		{
                            new_page();
                        }
             }

}

void hk_report::set_periodic(unsigned long when, const hk_string& what="",bool newpage=false)
{
   p_includeperiodic=when;
   p_includeperiodichk_string=what;
   p_newpage_on_periodic=newpage;
}


hk_string hk_report::replace(const hk_string& where)
{
  hk_string p_buffer;
// p_buffer=hk_reportbase::replace(where);
  p_buffer=replace_all("%FONT%",where,font());
 p_buffer=replace_all("%FONTSIZE%",p_buffer,fontsize());

 time_t 	p_nowtime=time(NULL);
 time_t		p_timediff=p_nowtime - p_currenttime;
 char * m=new char[50];
 sprintf(m,"%ld",p_timediff);

 p_buffer=replace_all("%PTIME%",p_buffer,m);
 delete[]m;
 return p_buffer;
}


hk_string hk_report::font(void)
{
 if (p_font.size()==0) return "Courier";
 	else return p_font;
}

hk_string hk_report::fontsize(void)
{
  if (p_fontsize.size()==0) return "10";
   return p_fontsize;
}


void  hk_report::set_filebegin(const hk_string& b)
{
  p_filebegin=b;
}


void	hk_report::set_fileend(const hk_string& e)
{
  p_fileend=e;
}



unsigned long hk_report::rownumber(void)
{
 if (datasource()!=NULL) return datasource()->row_position();
 else return 0;
}


bool hk_report::save_report(const hk_string& n,bool ask)
{
  if (database()==NULL)
  {
   show_warningmessage(hk_translate("Form error: No database defined!"));
   return false;

  }
  if (n.size()>0) set_name(n);

  reset_has_changed();
  if (name().size()==0) if (!ask_name())return false;
 ofstream* p_save=database()->savestream(name(),ft_report,ask);
 if (p_save==NULL) return false;
 savedata(*p_save);
 delete p_save;
 return true;

}

void  hk_report::savedata(ofstream& s)
{
start_mastertag( s,"MAINREPORT");
hk_string version="0.1";
	set_tagvalue(s,"VERSION",version);
	hk_dsvisible::savedata(s);
	set_tagvalue(s,"REPORTBEGIN",p_begin);
	set_tagvalue(s,"REPORTEND",p_end);
	set_tagvalue(s,"FILEBEGIN",p_filebegin);
	set_tagvalue(s,"FILEEND",p_fileend);
	set_tagvalue(s,"FILENAME",p_originalfilename);
	set_tagvalue(s,"FILEEXTENSION",p_fileextension);
	set_tagvalue(s,"MULTIPLEFILES",p_multiplefiles);
	set_tagvalue(s,"PAGEDELIMITER",p_pagedelimiter);
	set_tagvalue(s,"PAGENAME",p_pagename);
	set_tagvalue(s,"INCLUDEPERIODIChk_string",p_includeperiodichk_string);
	set_tagvalue(s,"INCLUDEPERIODIC",p_includeperiodic);
	set_tagvalue(s,"NEWPAGEPERIODIC",p_newpage_on_periodic);
	set_tagvalue(s,"STARTPAGENUMBER",p_startpagenumber);
	set_tagvalue(s,"SECTIONS",(unsigned long)p_sections.size());
/*        if (datasource()!=NULL)
        {
          set_tagvalue(s,"SQL",datasource()->sql());
          set_tagvalue(s,"TABLENAME",datasource()->name());
        } */
        if (datasource()!=NULL) datasource()->savedata(s);
hk_string recfunc="NONE";
 if (p_recodefunction!=NULL)
 if (p_recodefunction==recode_html) recfunc="HTML";
 else
   if (p_recodefunction==recode_postscript) recfunc="POSTSCRIPT";
    else recfunc="OTHER";
set_tagvalue(s,"RECODEFUNCTION",recfunc);
start_mastertag(s,"PAGEHEADSECTION");
p_pagehead->savedata(s);
end_mastertag(s,"PAGEHEADSECTION");
start_mastertag(s,"PAGEFOOTSECTION");
p_pagefoot->savedata(s);
end_mastertag(s,"PAGEFOOTSECTION");

start_mastertag(s,"SECTIONSECTION");
        vector<hk_reportsection*>::iterator it=p_sections.begin();
        while (it!=p_sections.end())
        {
         (*it)->savedata(s);
         it++;
        }
end_mastertag(s,"SECTIONSECTION");

end_mastertag(s,"MAINREPORT");
}

void hk_report::load_report(const hk_string& name)
{
  if (datasource()==NULL)
  {
     show_warningmessage(hk_translate("No datasource set in this report!"));
     return;
  }
 hk_string pn=name;
 if (pn.size()==0)pn=p_reportname;
 else p_reportname=pn;
 char buffer;
 pn+=".hk_report";
ifstream     p_load(pn.c_str(),ios::in);
  if (!p_load)
     {hk_string w=hk_translate("No such report: ");
       w+=pn;
       show_warningmessage(w);
       return;
     }
  hk_string file;

  while (!p_load.eof())
  {
   p_load.get(buffer);
   file+=buffer;

  }

 loaddata(file);

}
void  hk_report::loaddata(const hk_string& definition)
{
hk_string report;
if(get_tagvalue(definition, "MAINREPORT", report))
{
//	hk_reportbase::loaddata(definition);
	get_tagvalue(report,"REPORTBEGIN",p_begin);
	get_tagvalue(report,"REPORTEND",p_end);
	get_tagvalue(report,"FILEBEGIN",p_filebegin);
	get_tagvalue(report,"FILEEND",p_fileend);
	get_tagvalue(report,"FILENAME",p_originalfilename);
	get_tagvalue(report,"FILEEXTENSION",p_fileextension);
	get_tagvalue(report,"MULTIPLEFILES",p_multiplefiles);
	get_tagvalue(report,"PAGEDELIMITER",p_pagedelimiter);
	get_tagvalue(report,"PAGENAME",p_pagename);
	get_tagvalue(report,"INCLUDEPERIODIChk_string",p_includeperiodichk_string);
	get_tagvalue(report,"INCLUDEPERIODIC",p_includeperiodic);
	get_tagvalue(report,"NEWPAGEPERIODIC",p_newpage_on_periodic);
	get_tagvalue(report,"STARTPAGENUMBER",p_startpagenumber);
        hk_string ds;
        get_tagvalue(report,"DATASOURCE",ds);
        if (datasource()!=NULL)
        {
         datasource()->loaddata(ds);

        }
	unsigned long sections;
	get_tagvalue(report,"SECTIONS",sections);
	hk_string recfunc;
	get_tagvalue(report,"RECODEFUNCTION",recfunc);
  p_recodefunction=NULL;
 if (recfunc=="HTML")p_recodefunction=&recode_html;
 if (recfunc=="POSTSCRIPT")p_recodefunction=&recode_postscript;
 hk_string page;
        get_tagvalue(report,"PAGEHEADSECTION",page);
        p_pagehead->loaddata(page);
        get_tagvalue(report,"PAGEFOOTSECTION",page);
        p_pagefoot->loaddata(page);

 hk_string sectionsection;
       get_tagvalue(definition,"SECTIONSECTION",sectionsection) ;
       for (unsigned long i=1;i<=sections;i++)
       {
        hk_reportsection* section=new_section();
        hk_string s;get_tagvalue(sectionsection,"SECTION",s,i);
        section->loaddata(s);
       }

}

}



void hk_report::set_reportname(const hk_string& n)
{
if (n.size()>0) p_reportname=n;

}

hk_string hk_report::reportname(void)
{
 return p_reportname;
}

bool hk_report::set_presentationdatasource(long n, bool registerchange=true)
{
  hk_presentation::set_presentationdatasource(n,registerchange);
  vector<hk_reportsection*>::iterator it=p_sections.begin();
  while (it!= p_sections.end())
  {
    (*it)->set_presentationdatasource(n,registerchange);
    it++;
  }
  if (p_pagehead!=NULL)
  {
     p_pagehead->set_presentationdatasource(n,registerchange);
  }
  if (p_pagefoot!=NULL)
  {
     p_pagefoot->set_presentationdatasource(n,registerchange);
  }


}



