// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
#ifndef HK_REPORT
#define HK_REPORT
#include "hk_definitions.h"
#include "hk_presentation.h"
#include <list>
#include <vector>
//#include <hk_string>
#include <iostream>
#include <fstream>
#include <time.h>
class hk_reportsection;
class hk_reportdata;
class hk_report;
class hk_column;




/**
* With hk_report you can create reports to print all your database data in all userdefined, textbased formats, such as
* CVS, HTML, XML, Postscript etc. It has all functions you need to create reports in any kind and any format.
* See also the convinience classes @ref hk_reporthtml and @ref hk_reportcvs.
* @short basic class to create reports
*@version $Revision: 1.7 $
* @author Horst Knorr (hk_classes@knorrnet.de)

*

*in addition to the global variables defined in @ref hk_reportbase you can use the variable
*
<pre>%PTIME%        the time in seconds used to create the report output.
</pre>

*/



class hk_report :public hk_presentation
{

	public:
	 	  hk_report(void);
virtual	 	  ~hk_report(void);
/**
* Sets the filename of the report. If you dont give the report a name the report will be printed at
* the standard output.
*@see set_fileextension
*@see set_pagenamecolumn
*/
void		  set_filename(const hk_string&);
hk_string 	  filename(void);
/**
* Here you can set a hk_string which will be shown at the very beginning of the report (report header)
*/
void		  set_begin(const hk_string&);
hk_string	  begin(void);
/**
* Here you can set a hk_string which will be shown at the very end of the report (report footer)
*/
void		  set_end(const hk_string&);
hk_string	  end(void);
/**
*a report section contains the displayed data, is able to group data (sum, count, average etc.)
*and is the major feature of a report. You have to define at least one section for a report
*/
virtual hk_reportsection* new_section(void);
 hk_reportsection* new_section_at(unsigned int);
 hk_reportsection* section_at(unsigned int);
void		  delete_section(hk_reportsection*);
void		  delete_section(unsigned int);
/**
*After you have defined the report with @ref new_section etc. you can start the report
*by calling this function.
*/
bool		  execute(void);
/**
*If the data has to be recoded (i.e. ASCII to html -> &uuml;) you can write a function and
*set it with set_recodefunction
*/
recodefunctiontype* recodefunction(void){return p_recodefunction;}
void		set_recodefunction(recodefunctiontype* r){p_recodefunction=r;}
	void	new_page(void);
/**
*The page head will be printed on top of every page
*/
hk_reportsection* page_head(void){return p_pagehead;}
/**
*The page head will be printed on top of every page
*/
hk_reportsection* page_foot(void){return p_pagefoot;}
/**
* The pagedelimiter is a hk_string sequence to be sent between to pages (i.e. an escape
*sequence for the printer)
*/
void 		set_pagedelimiter(const hk_string& d){p_pagedelimiter=d;}
/**
*A list of all the sections
*/
vector<hk_reportsection*>* sections(void){return &p_sections;}
/**
*If a report has more than one page and this function is set to true
*every page will be stored in a separate file.
*/
virtual void 	set_multiplefiles(bool m=true){p_multiplefiles=m;}
/**
*The file name will be completed with this extension
*/
void 		set_fileextension(const hk_string& ext){p_fileextension=ext;}
/**
*Set the pagenumber if the first page should not start with 1.
*/
void		set_firstpagenumber(long int f);
/**
*You have different possibilities to name the report files. First, with @ref set_filename @ref set_fileextension
*Second, you can have the the filename in a column of your datasource. Give the name of the column with this function
*and then it will look up the name in this column. It is your responsibility to make sure that every filename is unique
*/
void 		set_pagenamecolumn(const hk_string& c);
/**
*will be set at the beginning of a file. If a report has multiple files b will be written in each file
*/
void		set_filebegin(const hk_string& b);
/**
*will be set at the end of a file. If a report has multiple files e will be written in each file
*/
void		set_fileend(const hk_string& e);

/**
*A hk_string that will be printed whenever a internal counter will get as big or greater than "when"
*To define "what counts what" use @ref hk_reportbase::set_counts_as
*Whenever the event of "counting" occurs by calling @ref row_count.
*Example: You want to print 50 lines per page, your header is 3 lines wide and your data isone line
*per data row set your page head to "3" and your datasection to "1"
*/
void 		set_periodic(unsigned long when, const hk_string& what="",bool newpage=false);
/**
*If the report will be printed to the standard output the outputstream is cout, else a textstream.
*/
ostream* 	outputstream(void){return p_output;}
/**
*A report can contain a subreport. If this report is a subreport this is its link to its master report
*Usually you dont have to call this function
*/

void		set_masterreport(hk_report* m){p_masterreport=m;}
/**
*This function does the counting job for @ref set_periodic and will be called automaticallly
*/
void		rowcount(long);
virtual	hk_string font(void);
virtual hk_string fontsize(void);
/**
*the number of the page. To get more than one page you have to use either
* @ref hk_reportsection::set_new_page_after_section
*or @ref hk_report::set_periodic
*/

long pagenumber(void){return p_pagenumber;}
long absolutepagenumber(void){return p_absolutepagenumber;}
unsigned long rownumber(void);

virtual	bool		set_presentationdatasource(long n, bool registerchange=true);


virtual bool save_report(const hk_string& name="",bool ask=true);
virtual void load_report(const hk_string& name="");
virtual void  savedata(ofstream& s);
virtual void  loaddata(const hk_string& definition);
void	set_reportname(const hk_string& r);
hk_string reportname(void);
	protected:
virtual hk_string replace(const hk_string& where);
//virtual bool widget_specific_row_change();
virtual bool datasource_disable(void);
virtual bool before_columns_deleted(void);
        bool 		p_multiplefiles;
        vector<hk_reportsection*> p_sections;
	private:

	void	new_outputstream(void);
void		set_pagenumber(long int f);
        hk_string 		p_begin;
        hk_string 		p_end;
        hk_string		p_filebegin;
        hk_string		p_fileend;
        hk_string		p_filename;
        hk_string		p_originalfilename;
        hk_string		p_fileextension;
        hk_string 		p_reportname;
        recodefunctiontype*
        		p_recodefunction;
        ostream*      p_output;
   	ofstream*     p_filestream;

   	hk_reportsection* p_pagehead;
   	hk_reportsection* p_pagefoot;
   	hk_string 		  p_pagedelimiter;
   	bool		p_lastpage;
        hk_column*	p_pagenamecolumn;
        hk_string		p_pagename;
        hk_string		p_includeperiodichk_string;
        unsigned long   p_includeperiodic;
        hk_report*	p_masterreport;
         long   	p_rowcounter;
         bool 		p_newpage_on_periodic;
         time_t 	p_currenttime;
         long 		p_pagenumber;
         long 		p_startpagenumber;
         long		p_absolutepagenumber;
}
;

#endif
