// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.3 $
#include "hk_reportpostscript.h"
#include "hk_reportsection.h"
#include "hk_reportdata.h"


//************************************
//*     hk_reportpostscript begin    *
//************************************

hk_reportpostscript::hk_reportpostscript(void)
{
   set_fileextension(".ps");
   set_pageformat(A4);
   set_orientation();
   set_pagedelimiter("showpage\ngrestore\nset_pageboundaries\n");
   p_defaultbeforedata="%XPOS% zmm %YPOS% zmm %WIDTH% zmm %HEIGHT% zmm (%ALIGN%) %FONTSIZE%  set_variables\n(";
   p_defaultafterdata=") /%FONT% set_all\n";
   set_borders();
   set_recodefunction(&recode_postscript);
   p_top=p_bottom=p_left=p_right=p_luro=p_loru=false;
   configure_page();
}


hk_reportpostscript::~hk_reportpostscript(void)
{

}

int 	hk_reportpostscript::formatwidth(pageformattype t)
{
int res;
            switch(t)
            {
             case A0  :	res=8410;
             		break;
             case A1  :	res=5940;
             		break;
             case A2  :	res=4200;
             		break;
             case A3  :	res=2970;
             		break;
             case A4  :	res=2100;
             		break;
             case A5  :	res=1480;
             		break;
             case A6  :	res=1050;
             		break;
             case A7  :	res=740;
             		break;
             case A8  :	res=520;
             		break;
             case A9  :	res=370;
             		break;
             case A10 :	res=260;
             		break;
             case letter :res=2160;
             		break;
             case legal :res=2160;
             		break;
             case executive :res=1900;
             		break;
             case tabloid :res=2790;
             		break;
             default  :res=2100;
            }
return res;
}


int 	hk_reportpostscript::formatheight(pageformattype t)
{
int res;
            switch(t)
            {
             case A0  :	res=11890;
             		break;
             case A1  :	res=8410;
             		break;
             case A2  :	res=5940;
             		break;
             case A3  :	res=4200;
             		break;
             case A4  :	res=2970;
             		break;
             case A5  :	res=2100;
             		break;
             case A6  :	res=1480;
             		break;
             case A7  :	res=1050;
             		break;
             case A8  :	res=740;
             		break;
             case A9  :	res=520;
             		break;
             case A10 :	res=370;
             		break;
             case letter :res=2790;
             		break;
             case legal :res=3560;
             		break;
             case executive :res=2540;
             		break;
             case tabloid :res=4320;
             		break;
             default  :res=2970;
            }
return res;
}



void hk_reportpostscript::set_pageformat(pageformattype t=A4)
{
   if (t!=userdefined)
   {
     p_pageformat=t;
     p_pagewidth=formatwidth(p_pageformat);
     p_pageheight=formatheight(p_pageformat);
     if (p_orientation==landscape)
     {
        int b=p_pagewidth;
        p_pagewidth=p_pageheight;
        p_pageheight=b;
     }
   }
   configure_page();
}

void hk_reportpostscript::set_pageformat(int width, int height)
{
 p_pageformat=userdefined;
 p_pagewidth=width;
 p_pageheight=height;
 configure_page();

}

void hk_reportpostscript::set_borders(int left, int right, int top, int bottom)
{
 p_borderleft=left;
 p_borderright=right;
 p_borderbottom=bottom;
 p_bordertop=top;
 configure_page();
}

int	hk_reportpostscript::from_ps(int p)
{

  return 254*p/72;
}


int 	hk_reportpostscript::to_ps(int p)
{

   return 72*p/254;
}


hk_reportsection* hk_reportpostscript::new_section(void)
{
  hk_reportsection* res=hk_report::new_section();
//  p_sections.insert(p_sections.end(),res);
   res->set_default_beforecolumnvalue(p_defaultbeforedata);
   res->set_default_aftercolumnvalue(p_defaultafterdata);
   res->set_sectionend("/offset offset %COUNTS_AS% zmm  sub def\n");
   res->set_sectioncountfunction(&recount_postscript);
   res->set_defaultdataconfigurefunction(&configure_postscriptdata);



 return res;
}


void hk_reportpostscript::configure_page(void)
{
   p_filehead="%!PS-ADOBE-3.0\n%CREATED WITH HK_CLASSES\n%<c> 2000 by Horst Knorr <hk_classes@knorrnet.de>\n%\n%\n%DEFINITIONS\n%\n\
/set_clippath\n{\nnewpath\n\tx leftborder add  y  moveto\n\twidth 0 rlineto\n\t0 height rlineto\n\
   \twidth neg 0 rlineto\n\tclosepath clip\n} def\n\n \
/position\n{\n\tdup hk_stringwidth pop\n\t/w exch def\n\talign (RIGHT) eq\n\t{/nx x width add leftborder add w sub def}\
{\n\talign (CENTER) eq\n\t{/nx x width 2 div add leftborder add w 2 div sub def}\n\t{/nx x leftborder add def\n\t}\
\n\t   	ifelse\n\t}\n\tifelse\n} def\n\n\
/set_textposition\n{\n\tposition\n\tnx y fontsize 5 div add moveto\n} def\n\n\
/set_font\n{\n\t/wantedfont exch def\n\twantedfont /myfont isolatinencoding ReEncode\n\t/myfont\
                \n\tfindfont\n\tfontsize scalefont\n\tsetfont\n} def\n\n\
/set_variables\n{\n\t/fontsize exch def\n\t/align exch def\n\t/height exch def\n\t/width exch def\n\t/y exch def\n\t/x exch def\n\t/yrel offset y sub fontsize sub def\n\t/y offset y sub height sub def\n} def\n\n\
/set_all\n{\n\tgsave\n\tset_clippath\n\tset_font\n\tset_textposition\n\tshow\n\tgrestore\n} def\n\n\
   \n/set_pageboundaries\n{\n\tgsave\n\t/x 0 def\n\t/y bottomborder def\n\t/width pagewidth leftborder sub rightborder sub def\n\t/height pageheight topborder sub bottomborder sub def\n\
   \t/offset pageheight topborder sub def\n\tset_clippath\n";

 if (p_top)p_filehead+="\tpaint_top\n";
 if (p_bottom)p_filehead+="\tpaint_bottom\n";
 if (p_left)p_filehead+="\tpaint_left\n";
 if (p_right)p_filehead+="\tpaint_right\n";
 if (p_luro)p_filehead+="\tpaint_diagonal_luro\n";
 if (p_loru)p_filehead+="\tpaint_diagonal_loru\n";

   p_filehead+="\n\t} def\
   \n [ /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n \
 /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n \
 /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n \
 /.notdef /.notdef /space /exclam /quotedbl /numbersign /dollar /percent /ampersand /quoteright\n \
 /parenleft /parenright /asterisk /plus /comma /hyphen /period /slash /zero /one\n \
 /two /three /four /five /six /seven /eight /nine /colon /semicolon\n \
 /less /equal /greater /question /at /A /B /C /D /E\n \
 /F /G /H /I /J /K /L /M /N /O  /P /Q /R /S /T /U /V /W /X /Y\n \
 /Z /bracketleft /backslash /bracketright /asciicircum /underscore /quoteleft /a /b /c\n \
 /d /e /f /g /h /i /j /k /l /m\n \
 /n /o /p /q /r /s /t /u /v /w\n \
 /x /y /z /braceleft /bar /braceright /asciitilde /.notdef /.notdef /.notdef\n \
 /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n \
 /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n \
 /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n \
 /space /exclamdown /cent /sterling /currency /yen /brokenbar /section /dieresis /copyright\n \
 /ordfeminine /guillemotleft /logicalnot /hyphen /registered /macron /degree /plusminus /twosuperior /threesuperior\n \
 /acute /mu /paragraph /periodcentered /cedilla /onesuperior /ordmasculine /guillemotright /onequarter /onehalf\n \
 /threequarters /questiondown /Agrave /Aacute /Acircumflex /Atilde /Adieresis /Aring /AE /Ccedilla\n \
 /Egrave /Eacute /Ecircumflex /Edieresis /Igrave /Iacute /Icircumflex /Idieresis /Eth /Ntilde\n \
 /Ograve /Oacute /Ocircumflex /Otilde /Odieresis /multiply /Oslash /Ugrave /Uacute /Ucircumflex\n \
 /Udieresis /Yacute /Thorn /germandbls /agrave /aacute /acircumflex /atilde /adieresis /aring\n \
 /ae /ccedilla /egrave /eacute /ecircumflex /edieresis /igrave /iacute /icircumflex /idieresis\n \
 /eth /ntilde /ograve /oacute /ocircumflex /otilde /odieresis /divide /oslash /ugrave\n \
 /uacute /ucircumflex /udieresis /yacute /thorn /ydieresis] /isolatinencoding exch def \n\n \
/ReEncode\n{\n\t/newencoding exch def\n\t/newfontname exch def\n\
   \t/basefontname exch def\n\t/basefontdict basefontname findfont def\n\
   \t/newfont basefontdict maxlength dict def\n\n\tbasefontdict\n\t{\
    \n\texch dup dup /FID ne exch /Encoding ne and \n\t{ exch newfont 3 1 roll put }\n\t{ pop pop } \
    \n\tifelse\n\t} forall\n\tnewfont /FontName newfontname put\n\tnewfont /Encoding newencoding put \
   \n\tnewfontname newfont definefont pop\n} def\n\n\
   /paint_top\n{\n\tnewpath\n\tx leftborder add   y height add moveto\n\twidth  0 rlineto\n\tstroke\n} def\n\n\n \
   /paint_bottom\n{\n\tnewpath\n\tx leftborder add  y   moveto\n\twidth  0 rlineto\n\tstroke\n} def\n\n\n \
   /paint_left\n{\n\tnewpath\n\tx leftborder add  y   moveto\n\t0  height rlineto\n\tstroke\n} def\n\n\n \
   /paint_right\n{\n\tnewpath\n\tx leftborder add width add y  moveto\n\t0 height rlineto\n\tstroke\n} def\n\n\n \
   /paint_diagonal_loru\n{\n\tnewpath\n\tx leftborder add  y  height add moveto\n\twidth height neg rlineto\n\tstroke\n} def\n\n\n \
   /paint_diagonal_luro\n{\n\tnewpath\n\tx leftborder add  y  moveto\n\twidth height rlineto\n\tstroke\n} def\n\n\n \
   \n/zmm\n{\n\t72 mul 254 div\n} def\n\n\n \
   /pagehead\n{\n\tgrestore\n\tgsave\n\t/x 0 def\n\t/y pageheight topborder sub def\n\t/height topborder def\n\n \
   \t/width pagewidth leftborder sub rightborder sub def\n\tset_clippath\n\t/offset pageheight def\n} def\n\n\n \
   /pagefoot\n{\n\tgrestore\n\tgsave\n\t/x 0 def\n\t/y 0 def\n\t/height bottomborder def\n\n \
   \t/width pagewidth leftborder sub rightborder sub def\n\tset_clippath\n\t/offset bottomborder  def\n} def\n\n\
/proc\n\t{\n\tposition\n\tnx yrel moveto show\n\t/yrel yrel fontsize sub def\n\t} def\n\n\
/wordbreak ( ) def\n\n\
/BreakIntoLines\n\t{\n\tgsave\n\tset_clippath\n\tset_font\n\t/nextword () def\n\t/texthk_string exch def\n\t\
/breakwidth wordbreak hk_stringwidth pop def\n\t/curwidth 0 def\n\t/lastwordbreak 0 def\n\t/startchar 0 def\n\t\
/restoftext texthk_string def\n\t{  %loop begin\n\trestoftext wordbreak search\n\t{\n\t/nextword exch def pop\
\n\t/restoftext exch def\n\t/wordwidth nextword hk_stringwidth pop def\n\tcurwidth wordwidth add width gt\n\t\{\
\n\ttexthk_string startchar\n\tlastwordbreak startchar sub\n\tgetinterval proc\n\t/startchar lastwordbreak def\
\n\t/curwidth wordwidth breakwidth add def\n\t}\n\t{\n\t/curwidth curwidth wordwidth add breakwidth add def\n\t}\
ifelse\n\t/lastwordbreak lastwordbreak nextword length add 1 add def\n\t}\n\t{\n\t/restoftext exch def exit\n\t}\
\n\tifelse\n\t} loop\n\t/wordwidth nextword hk_stringwidth pop def\n\t/lastchar texthk_string length def\
\n\t/restlength  lastchar startchar sub  def\n\ttexthk_string startchar restlength\n\tgetinterval\n\t/txt exch def\
\n\ttxt hk_stringwidth pop width gt\n\t{\n\ttexthk_string startchar lastwordbreak startchar sub getinterval\
\n\tproc\n\trestoftext\n\tproc\n\t}\n\t{\n\ttxt\n\tproc\n\t}\n\tifelse\n\tgrestore\n}def\n\
\n%PROGRAMM START\n \n\n";
   char* m=new char[60];
   hk_string def;
  def+="pagehead\n";
   def+=p_defaultbeforedata;
   page_head()->set_default_beforecolumnvalue(def);
   def="pagefoot\n";
   def+=p_defaultbeforedata;
   page_foot()->set_default_beforecolumnvalue(def);
   def=p_defaultafterdata+"grestore\nset_pageboundaries\n";
   page_head()->set_default_aftercolumnvalue(def);
   page_foot()->set_default_aftercolumnvalue(def);




   hk_string h=p_filehead;
   if ((p_orientation==landscape)&&p_pageformat!=userdefined)
   {

    sprintf(m,"%d",p_pageheight);
    h+=m;
    h+=" zmm 0 translate\n90 rotate\n\n";
   }

   h+="\n/pagewidth ";
   hk_string d=" zmm def\n";
   sprintf(m,"%d",p_pagewidth);
   h+=m;
   h+=d+"/pageheight ";

   sprintf(m,"%d",p_pageheight);
   h+=m;
   h+=d+"/leftborder ";

   sprintf(m,"%d",p_borderleft);
   h+=m;
   h+=d+"/rightborder ";

   sprintf(m,"%d",p_borderright);
   h+=m;
   h+=d+"/topborder ";

   sprintf(m,"%d",p_bordertop);
   h+=m;
   h+=d+"/bottomborder ";

   sprintf(m,"%d",p_borderbottom);
   h+=m;
   h+=d+"set_pageboundaries\n ";

   set_filebegin(h);
   set_periodic(p_pageheight-p_borderbottom-p_bordertop,"",true);

   delete[] m;


}


void	hk_reportpostscript::set_topline(bool l=true)
{
    p_top=l;
    configure_page();
}
void	hk_reportpostscript::set_bottomline(bool l=true)
{
    p_bottom=l;
    configure_page();

}
void	hk_reportpostscript::set_leftline(bool l=true)
{
    p_left=l;
    configure_page();

}
void	hk_reportpostscript::set_rightline(bool l=true)
{
    p_right=l;
    configure_page();

}

void	hk_reportpostscript::set_diagonalluro(bool l=true)
{
   p_luro=l;
    configure_page();
}

void	hk_reportpostscript::set_diagonalloru(bool l=true)
{
  p_loru=l;
  configure_page();
}


void	hk_reportpostscript::set_frame(bool l=true)
{
    p_top=l;
    p_bottom=l;
    p_left=l;
    p_right=l;
    configure_page();
}


void hk_reportpostscript::set_orientation(orientationtype o=portrait)
{
	p_orientation=o;
	set_pageformat(p_pageformat);
}
