// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.24 $
#include <hk_visible.h>
#include <hk_presentation.h>
#include <hk_form.h>
#include "hk_report.h"

hk_visible::hk_visible(hk_presentation* p):hk_class()
{
#ifdef HK_DEBUG
hkdebug("hk_visible::hk_visible");
#endif
	p_visibletype=other;
	p_presentation=p;
	p_form=dynamic_cast <hk_form*>(p);
	p_report=dynamic_cast <hk_report*>(p);
	p_dynamicheight=p_dynamicwidth=true;
	set_size(0,0,1000,300,false,false,false);
	p_setcoordinates=p_setwidgetcoordinates=false;
	set_font("Courier",12,false);
	p_neverregisterchange=false;
	p_vupn=-1;
	p_counts_as=0;
        p_dateformat=p_defaultdateformat;
        p_timeformat=p_defaulttimeformat;
        p_datetimeformat=p_defaultdatetimeformat;

}



hk_visible::~hk_visible(void)
{
#ifdef HK_DEBUG
hkdebug("hk_visible::destructor");
#endif


}



void hk_visible::set_size(unsigned int x, unsigned int y,unsigned int w, unsigned int h, bool dynamicwidth,bool dynamicheight, bool registerchange)
{
#ifdef HK_DEBUG
hkdebug("hk_visible:set_size");
#endif
   if (p_setcoordinates==true)
   {
#ifdef HK_DEBUG
hkdebug("hk_visible:set_size psetcoordinates== true !!!");
#endif
    return;
   }
   p_setcoordinates=true;
   unsigned int ppx=x;
   unsigned int ppy=y;
   unsigned int ppw=w;
   unsigned int pph=h;
    p_dynamicheight=dynamicheight;
    p_dynamicwidth=dynamicwidth;
   if (p_form!=NULL)
   {
    if (ppw<500)ppw=500;
    if (pph<500)pph=500;
    if (ppx>9000)ppx=9000;
    if (ppx+ppw>10000)ppw=1000;
    if (ppy>9000)ppy=9000;
    if (ppy+pph>10000)pph=1000;

    ppx=p_form->relativ2horizontal(ppx);
    ppy=p_form->relativ2vertical(ppy);
    pph=p_form->relativ2vertical(pph,p_dynamicheight);
    ppw=p_form->relativ2horizontal(ppw,p_dynamicwidth);


   }
   if (p_setwidgetcoordinates)
   {
	#ifdef HK_DEBUG
	hkdebug("hk_visible:set_size p_setwidgetcoordinates ==true");
	#endif
      	p_x=x;
   	p_y=y;
   	p_width=w;
   	p_height=h;
   }
   else
   if(widget_specific_coordinates(ppx,ppy,ppw,pph))
   {
   	p_x=x;
   	p_y=y;
   	p_width=w;
   	p_height=h;
    if (p_form!=NULL) p_form->widget_specific_fieldresize(this);
   }
  has_changed(registerchange);
 p_setcoordinates=false;
}


void hk_visible::set_x(unsigned int px, bool registerchange)
{
  set_size(px,p_y,p_width,p_height,registerchange);
}


void hk_visible::set_y(unsigned int py, bool registerchange)
{
  set_size(p_x,py,p_width,p_height,registerchange);
}


void hk_visible::set_width(unsigned int pwidth, bool registerchange)
{
  set_size(p_x,p_y,pwidth,p_height,registerchange);
}

void hk_visible::set_size(unsigned int width, unsigned int height, bool registerchange)
{
   set_size(p_x,p_y,width,height,registerchange);
}

void hk_visible::set_position(unsigned int x, unsigned int y, bool registerchange)
{
   set_size(x,y,p_width,p_height,registerchange);
}

void hk_visible::set_height(unsigned int pheight, bool registerchange)
{
  set_size(p_x,p_y,p_width,pheight,registerchange);
}


unsigned int hk_visible::x()
{
	return p_x;
}


unsigned int hk_visible::y()
{
	return p_y;
}


unsigned int hk_visible::width()
{
	return p_width;
}


unsigned int hk_visible::height()
{
	return p_height;
}

bool    hk_visible::widget_specific_coordinates(unsigned int px,unsigned int py,unsigned int pwidth,unsigned int pheight)
{
	return true;
}



void 	hk_visible::widget_specific_font_changed(void)
{

}



void  hk_visible::widget_specific_form_resizes(void)
{
      set_size(p_x,p_y,p_width,p_height);
}


bool	hk_visible::dynamic_formwidth(void)
{
  return p_dynamicwidth;
}

bool	hk_visible::dynamic_formheight(void)
{
   return p_dynamicheight;
}

void hk_visible::set_label(const hk_string& l, bool registerchange)
{
#ifdef HK_DEBUG
hkdebug("hk_visible::label(hk_string)");
#endif
 p_label=l;
 has_changed(registerchange);
 widget_specific_label_changed();
}

hk_string hk_visible::label(void)
{

  return p_label;
}


void hk_visible::savedata(ofstream& s )
{
#ifdef HK_DEBUG
hkdebug("hk_visible::savedata");
#endif
  hk_string tag="HK_VISIBLE";
  start_mastertag(s,tag);
  hk_string v="";
  switch (p_visibletype)
  {
  case   textlabel     	:  v="TEXTLABEL";break;
  case button     	:  v="BUTTON";break;
  case rowselector     	:  v="SELECTOR";break;
  case boolean     	:  v="BOOLEAN";break;
  case lineedit     	:  v="LINEEDIT";break;
  case memo     	:  v="MEMO";break;
  case combobox     	:  v="COMBOBOX";break;
  case grid     	:  v="GRID";break;
  case form     	:  v="FORM";break;
  default               : v="UNKNOWN";
  } ;
  set_tagvalue(s,"VISIBLETYPE",v);
  set_tagvalue(s,"LABEL",p_label);
  set_tagvalue(s,"DYNAMIC_WIDTH",p_dynamicwidth);
  set_tagvalue(s,"DYNAMIC_HEIGHT",p_dynamicheight);
  set_tagvalue(s,"X",(unsigned long ) p_x);
  set_tagvalue(s,"Y",(unsigned long ) p_y);
  set_tagvalue(s,"WIDTH",(unsigned long ) p_width);
  set_tagvalue(s,"HEIGHT",(unsigned long ) p_height);
  set_tagvalue(s,"FONT",p_font);
  set_tagvalue(s,"FONTSIZE",p_fontsize);
  set_tagvalue(s,"VUPN",p_vupn);
  set_tagvalue(s,"COUNTS_AS",p_counts_as);

  end_mastertag(s,tag);

}

void hk_visible::loaddata(const hk_string& definition)
{
#ifdef HK_DEBUG
hkdebug("hk_visible::loaddata");
#endif
  hk_string v;
  get_tagvalue(definition,"VISIBLETYPE",v);
  get_tagvalue(definition,"LABEL",p_label);
  set_label(p_label);
  get_tagvalue(definition,"DYNAMIC_WIDTH",p_dynamicwidth);
  get_tagvalue(definition,"DYNAMIC_HEIGHT",p_dynamicheight);
  get_tagvalue(definition,"X",(unsigned long ) p_x);
  get_tagvalue(definition,"Y",(unsigned long ) p_y);
  get_tagvalue(definition,"WIDTH",(unsigned long ) p_width);
  get_tagvalue(definition,"HEIGHT",(unsigned long ) p_height);
  get_tagvalue(definition,"FONT",p_font);
  get_tagvalue(definition,"COUNTS_AS",p_counts_as);
  long int p=-1;
  if (get_tagvalue(definition,"VUPN",p))p_vupn=p;
  long int pfs;
  get_tagvalue(definition,"FONTSIZE",pfs);
  set_font(p_font,pfs);
  set_size(p_x,p_y,p_width,p_height);
}

void hk_visible::has_changed(bool registerchange)
{
#ifdef HK_DEBUG
hkdebug("hk_visible::has_changed(bool registerchange)");
#endif
  if ( (!registerchange)||(p_presentation==NULL)||p_neverregisterchange) return;

  p_presentation->set_has_changed();
}

void hk_visible::set_font(const hk_string& font,int size,bool registerchange)
{
   p_font=font;
   p_fontsizeint=size;
   char* m=new char[50];
  sprintf(m,"%d",size);
  p_fontsize=m;
  delete[] m;
  has_changed(registerchange);
  widget_specific_font_changed();
}

void	hk_visible::widget_specific_label_changed(void)
{

}


void hk_visible::set_neverregisterchange(bool n)
{
   p_neverregisterchange=n;
}


void hk_visible::set_datetimeformat(const hk_string& datetime,const hk_string& date, const hk_string&  time)
{
   p_datetimeformat=datetime;
   p_dateformat=date;
   p_timeformat=time;
}

hk_string hk_visible::datetimeformat(void)
{
   return p_datetimeformat;
}

hk_string hk_visible::dateformat(void)
{
   return p_dateformat;
}

hk_string hk_visible::timeformat(void)
{
   return p_timeformat;
}



unsigned long int hk_visible::counts_as(void)
{
  return p_counts_as;
}

void hk_visible::set_counts_as(unsigned long int c)
{
  p_counts_as=c;

}
