// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
#ifndef HK_VISIBLE
#define HK_VISIBLE

#include <hk_definitions.h>
#include <hk_class.h>
class hk_presentation;
class hk_form;
class hk_report;
/**
*
*@short base class for visible widgets
*@version $Revision: 1.17 $
*@author Horst Knorr (hk_classes@knorrnet.de)
*
*base class for visible widgets.
*/
class hk_visible:public hk_class

{
	friend class hk_presentation;
	friend class hk_form;
	public:
  			hk_visible(hk_presentation* p=NULL);
	virtual		~hk_visible(void);
/**
* see @ref set_size
*/
  	void		set_x(unsigned int px, bool registerchange=true);
/**
* see @ref set_size
*/
  	void		set_y(unsigned int py, bool registerchange=true);
/**
* see @ref set_size
*/
  	void		set_width(unsigned int w, bool registerchange=true);
/**
* see @ref set_size
*/
  	void		set_height(unsigned int h, bool registerchange=true);
  	unsigned int	x(void);
  	unsigned int	y(void);
  	unsigned int    width(void);
  	unsigned int    height(void);
/**
*sets the size and the position of the object.  In forms the coordinates are in a range from 0 to 10000 (100.00%)
*/  	void		set_size(unsigned int x, unsigned int y,unsigned int w, unsigned int h, bool dynamicwidth=true,bool dynamicheight=true, bool registerchange=true);
	void		set_size(unsigned int width, unsigned int height, bool registerchange=true);
	void  		set_position(unsigned int x, unsigned int y, bool registerchange=true);
	bool		dynamic_formwidth(void);
	bool		dynamic_formheight(void);
/**
* Every visible object has a (static) label which will be displayed when necessary, i.e. a combobox usually needs
* a label
*/
	void		set_label(const hk_string& l, bool registerchange=true);
	hk_string		label(void);
 enum enum_visibletype {textlabel,button,rowselector,boolean,lineedit,memo,combobox,grid,form,other};
 enum_visibletype  type(void) {return p_visibletype;}
/**
*sets the font name and size to be used of some reportgenerators
*/
	void	   set_font(const hk_string& font,int size,bool registerchange=true);
virtual hk_string     font(void){return p_font;}
virtual hk_string     fontsize(void){return p_fontsize;}
virtual int	   fontsizeint(void){return p_fontsizeint;}
/**
* if a hk_visible object is part of a @ref hk_presentation (i.e. a form) and the size or the label
* changes the presentation will be informed of this change (and will be stored when closed).
* If you set n to true the presentation will not be informed
*/
	void set_neverregisterchange(bool n);
	bool neverregisterchange(void){return p_neverregisterchange;}
virtual void savedata(ofstream& s );
virtual void loaddata(const hk_string& definition);
      long 	presentationnumber(void){return p_vupn;}
void set_datetimeformat(const hk_string& datetime,const hk_string& date, const hk_string&  time);
hk_string datetimeformat(void);
hk_string dateformat(void);
hk_string timeformat(void);
void set_counts_as(unsigned long int c);
unsigned long int counts_as(void);

	protected:
virtual	void		widget_specific_form_resizes(void);
virtual void		widget_specific_font_changed(void);
virtual	void		widget_specific_label_changed(void);
virtual bool		widget_specific_coordinates(unsigned int px,unsigned int py,unsigned int pwidth,unsigned int pheight);
	void 		has_changed(bool registerchange);
	hk_presentation* 	p_presentation;
	hk_form*		p_form;
	hk_report*		p_report;
	bool		p_dynamicwidth;
	bool		p_dynamicheight;
	hk_string		p_label;
 	enum_visibletype p_visibletype;
	bool		p_setcoordinates;
	bool		p_setwidgetcoordinates;
        hk_string 		p_font;
        hk_string 		p_fontsize;
	int 		p_fontsizeint;


	private:
  	unsigned int	p_x,p_y,p_width,p_height;
	bool		p_neverregisterchange;
	long int 	p_vupn;//visual unique presentation number
	hk_string p_datetimeformat;
	hk_string p_dateformat;
	hk_string p_timeformat;
	unsigned long int p_counts_as;
};

#endif
