// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_mysqlclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
#include <hk_mysqldatasource.h>
#include <hk_mysqldatabase.h>
#include <hk_mysqlconnection.h>
#include <hk_mysqlcolumn.h>
#include <hk_mysqlactionquery.h>

hk_mysqldatasource::hk_mysqldatasource(hk_mysqldatabase* d,hk_presentation* p):hk_datasource(d,p)
{
#ifdef HK_DEBUG
hkdebug("hk_mysqldatasource::constructor");
#endif 
	p_mysqldatabase=d;
        p_rows=0;
        p_columns=NULL;
 	p_length=NULL;
 	p_debug_counter=0;
 	p_enabled=false;
 	p_actionquery=new hk_mysqlactionquery(d);
        p_true="1";
        p_false="0";

}


hk_mysqldatasource::~hk_mysqldatasource()
{
#ifdef HK_DEBUG
hkdebug("hk_mysqldatasource::destructor");
#endif 
  delete_data();
}


void hk_mysqldatasource::delete_data(void)
{
      	vector<struct_raw_data*>::iterator it=p_data.begin();
      	while(it!=p_data.end())
      	{
      	   struct_raw_data* datarow=*it;
      	   it++;
      	   if (columns()!=NULL) for (unsigned int k=0;k<columns()->size();k++)
      	   {
      	   delete[] (datarow[k]).data;
	   }
     	   delete[] datarow;
      	   datarow=NULL;
	   
      	}
      	p_data.erase(p_data.begin(),p_data.end());

}


hk_datasource::enum_datasourcetypes hk_mysqldatasource::type()
{
   return ds_table;
}



list<hk_column*>* hk_mysqldatasource::driver_specific_columns(void)
{
    if (p_columns==NULL&&type()==ds_table&&p_name.size()>0)
    {
       //create fields if necsessary only for tables i.e. in altermode
     p_result=mysql_list_fields(p_mysqldatabase->connection()->dbhandler(),p_name.c_str(),NULL);
     driver_specific_create_columns();
     mysql_free_result(p_result);

    }
    return p_columns;
}


bool hk_mysqldatasource::driver_specific_create_columns(void)
{
#ifdef HK_DEBUG
hkdebug("create_columns");
#endif 
   unsigned int num_fields,r;
   MYSQL_FIELD *fields;
  	if (p_result!=NULL)
	{
	 	clear_columnlist();
	 	p_columns=new list<hk_column*>;

	 	num_fields=mysql_num_fields(p_result);
	        r=0;
		int z = 0;
	       while ((fields=mysql_fetch_field(p_result)))
	       {
	                hk_mysqlcolumn *col= new hk_mysqlcolumn(this,p_true,p_false);
	                 #ifdef HK_DEBUG
	                 {
	                    char* m;
	                    char* n;
    				m= new char[50];
    				sprintf(m,"%ld",p_debug_counter++);
    				n=new char[strlen(m)+16];
    				strcpy(n,"Mysqlcolumn ");
    				strcat(n,m);
    				col->hkclassname(n);
    				hkdebug("New Column");
    				delete[] m;
    				delete[] n;
    			  }
                         #endif 
	                col->set_name(fields->name);
	                col->set_size(fields->length);
	                col->fieldnr(z++);
                        if (p_datetimeformat.size()>0) col->set_datetimeformat(p_datetimeformat.c_str());
   			if (p_dateformat.size()>0) col->set_dateformat(p_dateformat.c_str());
   			if (p_timeformat.size()>0) col->set_timeformat(p_timeformat.c_str());

	               switch(fields->type)
	                {
	                	case FIELD_TYPE_FLOAT :
	                					col->set_fieldtype(hk_column::smallfloatingfield) ;
	                					break;
	                	case FIELD_TYPE_DOUBLE:
	                					col->set_fieldtype(hk_column::floatingfield) ;
	                					break;
	                	case FIELD_TYPE_LONG:
	                	case FIELD_TYPE_INT24:
	                	case FIELD_TYPE_LONGLONG:       col->set_fieldtype(hk_column::integerfield);
	                					if (fields->flags & AUTO_INCREMENT_FLAG)
	                						col->set_fieldtype(hk_column::auto_incfield);
	                					break;
	                	case FIELD_TYPE_TINY:           col->set_fieldtype(hk_column::boolfield);
								break;
	                	case FIELD_TYPE_SHORT:           col->set_fieldtype(hk_column::smallintegerfield);
								break;


				case FIELD_TYPE_STRING:
	                	case FIELD_TYPE_VAR_STRING:
	                	case FIELD_TYPE_DECIMAL : 	col->set_fieldtype(hk_column::textfield);
	                					break;
	                	case FIELD_TYPE_DATETIME:
	                	case FIELD_TYPE_TIMESTAMP:	col->set_fieldtype(hk_column::datetimefield);
	                					break;
	                	case FIELD_TYPE_NEWDATE:
	                	case FIELD_TYPE_DATE:		col->set_fieldtype(hk_column::datefield);
	                					break;
	                	case FIELD_TYPE_TIME:		col->set_fieldtype(hk_column::timefield);
	                					break;
	                	case FIELD_TYPE_TINY_BLOB:
	                	case FIELD_TYPE_MEDIUM_BLOB:
	                	case FIELD_TYPE_LONG_BLOB:
	                	case FIELD_TYPE_BLOB:		if (fields->flags & BINARY_FLAG)
								col->set_fieldtype(hk_column::binaryfield);
								else
								col->set_fieldtype(hk_column::memofield);
	                					break;
	                		default:		col->set_fieldtype(hk_column::otherfield);


	                } //endswitch

	                if (fields->flags & PRI_KEY_FLAG)
	                  {
	                    col->set_primary(true);
	                    p_primary_key_used=true;
	                  }
	                if (fields->flags & NOT_NULL_FLAG) col->set_notnull(true);
	                p_columns->insert(p_columns->end(),col);

	       };//end while


	delete fields;
	return true;
     }
   return false;

}


bool hk_mysqldatasource::driver_specific_enable(void)
{
   int i;
#ifdef HK_DEBUG
hkdebug("driver_specific_enable");
#endif 
  if (!p_enabled)
  {
        if  (p_mysqldatabase==NULL)  return false;
        if  (!p_mysqldatabase->connection()->is_connected()) return false;
	i= mysql_query(p_mysqldatabase->connection()->dbhandler() ,p_sql.c_str() );
	if (i==0)
	{
		p_result=mysql_use_result(p_mysqldatabase->connection()->dbhandler());
		if (p_result==NULL) return false;
	 	p_rows=mysql_num_rows(p_result);
	 	unsigned int num_fields=mysql_num_fields(p_result);
	 	unsigned long zeile=0;
	 	while ((p_row=mysql_fetch_row(p_result)))
	 	{
	 	//Jede einzelne Zeile durchhoppeln
                #ifdef HK_DEBUG
                //hkdebug("Zeile", zeile++);
                #endif 
                zeile++;
                p_length=mysql_fetch_lengths(p_result);
                //neue Datenzeile erzeugen
                struct_raw_data* datarow=new struct_raw_data[num_fields];
	 	unsigned int spalte=0;
	 	while (spalte<num_fields)
	 	{
	 	   datarow[spalte].length=p_length[spalte];
	 	   char* data=new char[datarow[spalte].length];
	 	   #ifdef HK_DEBUG
//	 	   hkdebug("Spaltenlnge: ",(int)datarow[spalte].length);
	 	   #endif
	 	   for (unsigned int tk=0;tk<datarow[spalte].length;tk++) data[tk]=p_row[spalte][tk];
	 	   datarow[spalte].data=data;
	 	   #ifdef HK_DEBUG
//	 	   hkdebug("Spalte ",(int)spalte);
	 	   #endif 
	 	   spalte++;
	 	}
	 	//Daten speichern
	 	p_data.insert(p_data.end(),datarow);
	 	}
	 	driver_specific_create_columns();
	 	mysql_free_result(p_result);
	 	p_result=NULL;
	 	p_rows=zeile;
        	return true;
        }
         else return false;

 }
 return false;
}


bool hk_mysqldatasource::driver_specific_disable(void)
{
#ifdef HK_DEBUG
hkdebug("hk_mysqldatasource::driver_specific_disable");
#endif 
  delete_data();
  p_result=NULL;
  p_rows=0;
  p_counter=0;
  return true;

}


vector<struct_raw_data*>* hk_mysqldatasource::data()
{
 	return &p_data;
}



bool hk_mysqldatasource::driver_specific_goto_row(unsigned long r)
{
     #ifdef HK_DEBUG
     hkdebug("driver_specific_goto_row");
     #endif 

   if (p_enabled) if (r <p_rows) return true;
   return false;

}


unsigned long hk_mysqldatasource::driver_specific_max_rows(void)
{
     #ifdef HK_DEBUG
 //    hkdebug("driver_specific_max_rows");
     #endif
 return p_rows;
}


hk_column* hk_mysqldatasource::driver_specific_new_column(void)
{
     #ifdef HK_DEBUG
    hkdebug("driver_specific_new_column");
     #endif 
   hk_column* col=new hk_mysqlcolumn(this,p_true,p_false);

 return col;
}


bool hk_mysqldatasource::driver_specific_update_data(void)
{
 list<hk_column*>::iterator it;
 it=p_columns->begin();
 while (it!=p_columns->end())
 {
  hk_column* x=*it;
  if (x->has_changed())
  {

  	struct_raw_data*  data=(p_data[p_counter]);
  	data[x->fieldnumber()].length=x->changed_data()->length;
  	delete[] data[x->fieldnumber()].data;
  	char* buffer =new char[x->changed_data()->length];
  	memcpy(buffer,x->changed_data()->data,x->changed_data()->length);
  	data[x->fieldnumber()].data=buffer;

  }
  it++;
 }

 return true;
}


bool hk_mysqldatasource::driver_specific_delete_data_at(unsigned long position)
{

  	struct_raw_data*  data=(p_data[position]);

   	if (columns()!=NULL)
   	for (unsigned int k=0;k<columns()->size();k++)
   	{delete[] (data)[k].data;
   	}
        vector<struct_raw_data*>::iterator it=find(p_data.begin(),p_data.end(),data);
        p_data.erase(it);
        delete data;
        data=NULL;
        if (p_rows>0) p_rows--;

 return true;
}

bool hk_mysqldatasource::driver_specific_insert_data(void)
{

                struct_raw_data* datarow=new struct_raw_data[p_columns->size()];
                list<hk_column*>::iterator col_it;
                col_it=p_columns->begin();
	 	unsigned int spalte=0;
	 	while (spalte<p_columns->size())
	 	{
	 	   const struct_raw_data* changed_data=(*col_it)->changed_data();
	 	   unsigned long new_autoinc=mysql_insert_id(p_mysqldatabase->connection()->dbhandler());
	 	   if ((*col_it)->fieldtype()==hk_column::auto_incfield)
	 	   {
	 	   char* data=new char[100];
	 	   sprintf(data,"%ld",new_autoinc);
	 	   datarow[spalte].data=data;
	 	   datarow[spalte].length=strlen(data);
	 	   }
	 	   else
	 	   {
	 	   datarow[spalte].length=changed_data->length;;
	 	   char* data=new char[datarow[spalte].length];
	 	   #ifdef HK_DEBUG
	 	   hkdebug("Spaltenlnge: ",(int)datarow[spalte].length);
	 	   #endif 
	 	   for (unsigned int tk=0;tk<datarow[spalte].length;tk++) data[tk]=changed_data->data[tk];
	 	   datarow[spalte].data=data;
	 	   #ifdef HK_DEBUG
	 	   hkdebug("Spalte ",(int)spalte);
	 	   #endif 
	 	   }
	 	   spalte++;
	 	   col_it++;
	 	}
	 	//Daten speichern
	 	#ifdef HK_DEBUG
	 	/*hkdebug("Vor einfgen: ");
	 	dump_data();*/
	 	#endif 
	 	p_data.insert(p_data.end(),datarow);
	 	p_rows++;
	 	/*
	 	#ifdef HK_DEBUG
	 	hkdebug("Nach Einfgen");
	 	dump_data();


vector <struct_raw_data*>::iterator it;
cout<<endl<<"DUMPlokal"<<endl;
cout<<"===="<<endl;

  for (it=p_data.begin(); it!=p_data.end();it++)
  {
   for (unsigned int k=0;k<p_columns->size();k++)
   {
    for(unsigned int l=0;l<((*it)[k]).length;l++)if ((*it)->length>0)cout <<(*it)[k].data[l];
    cout <<" ";
   }
   cout <<endl;
  }
  cout<<endl;
cout <<"================="<<endl<<endl;


	 	#endif
*/
return true;
}


void hk_mysqldatasource::dump_data()
{

vector <struct_raw_data*>::iterator it;
cout<<endl<<"driverspecific DUMP"<<endl;
cout<<"===="<<endl;

  for (it=p_data.begin(); it!=p_data.end();it++)
  {
   for (unsigned int k=0;k<p_columns->size();k++)
   {
    for(unsigned int l=0;l<((*it)[k]).length;l++)if ((*it)->length>0)cout <<(*it)[k].data[l];
    cout <<" ";
   }
   cout <<endl;
  }
  cout<<endl;
cout <<"================="<<endl<<endl;


}


/*struct_raw_data* hk_mysqldatasource::raw_data_at(unsigned long col,unsigned long row)
{
vector <struct_raw_data*>::iterator it=p_data.begin();
   struct_raw_data* p= p_data[row];


   for (unsigned long k=0;k<row;k++)it++;
 struct_raw_data* dat=&((*it)[col]);
 return dat; */
/*
vector <struct_raw_data*>::iterator it;
int r=0;
  for (it=p_data.begin(); (it!=p_data.end());it++)r++;
   it--;
   p_d.length= ((*it)[col]).length;
   p_d.data=(*it)[col].data;


   return &p_d;

}
*/


